@@ -1,232 +1,225 @@
 # Copyright (c) ONNX Project Contributors
 #
 # SPDX-License-Identifier: Apache-2.0
-
-# pylint: disable=C0415,R0912,R0913,R0914,R0915
-"""
-Automates the generation of ONNX operators.
-"""
+"""Automates the generation of ONNX operators."""
+import difflib
 import importlib
 import inspect
 import keyword
 import os
+import pathlib
 import re
+import shutil
 import sys
 import textwrap
-from difflib import Differ
+from typing import Any
 
+import jinja2
 import numpy as np
 from sphinx.util import logging
 
 import onnx
 from onnx.backend.test.case.base import _Exporter
-from onnx.defs import OpSchema, get_all_schemas_with_history, get_schema
-from onnx.numpy_helper import to_array
-from onnx.onnx_cpp2py_export.defs import (  # pylint: disable=E1101,E0611,E0401
-    SchemaError,
+from onnx.defs import OpSchema
+
+REPO_DOCS_EXCLUDE = {
+    "Changelog-ml.md",
+    "Changelog.md",
+    "CIPipelines.md",
+    "CONTRIBUTING.md",
+    "Operators-ml.md",
+    "Operators.md",
+    "Relicensing.md",
+    "TestCoverage-ml.md",
+    "TestCoverage.md",
+}
+
+
+def _get_diff_template():
+    return jinja2.Template(
+        textwrap.dedent(
+            """
+            <div id="{{ div_name }}"></div>
+            <link rel="stylesheet" type="text/css" href="../_static/diff2html.min.css" />
+            <script type="text/javascript" src="../_static/diff2html-ui-slim.min.js"></script>
+            <script>
+            const diffString = `
+            --- a/{{ op_name }}{{ version1 }}
+            +++ b/{{ op_name }}{{ version2 }}
+            @@ -1 +1 @@
+            {{ diff_content }}
+            `;
+
+            document.addEventListener('DOMContentLoaded', function () {
+            var targetElement = document.getElementById('{{ div_name }}');
+            var configuration = {
+                drawFileList: true,
+                fileListToggle: false,
+                fileListStartVisible: false,
+                fileContentToggle: false,
+                matching: 'lines',
+                outputFormat: 'line-by-line',
+                synchronisedScroll: true,
+                highlight: true,
+                renderNothingWhenEmpty: false,
+            };
+            var diff2htmlUi = new Diff2HtmlUI(targetElement, diffString, configuration);
+            diff2htmlUi.draw();
+            diff2htmlUi.highlightCode();
+            });
+            </script>
+            """
+        ),
+        autoescape=True,
+    )
+
+
+def _get_ops_template():
+    return jinja2.Template(
+        """\
+{% for sch in schemas %}
+
+.. tag-diff-insert.
+
+(l-onnx-op{{sch.domain.lower().replace(".", "-")}}-{{sch.name.lower()}}-{{str(sch.since_version)}})=
+
+## {{format_name_with_domain(sch)}}
+
+### Version
+
+- **name**: [{{sch.name}} (GitHub)]({{build_doc_url(sch)}}{{sch.name}})
+- **domain**: `{% if sch.domain == '' %}main{% else %}{{sch.domain}}{% endif %}`
+- **since_version**: `{{sch.since_version}}`
+- **function**: `{{sch.has_function or sch.has_context_dependent_function}}`
+- **support_level**: `{{sch.support_level}}`
+- **shape inference**: `{{sch.has_type_and_shape_inference_function}}`
+
+{% if sch.support_level == OpSchema.SupportType.EXPERIMENTAL %}
+No versioning maintained for experimental ops.
+{% else %}
+This version of the operator has been {% if
+sch.deprecated %}deprecated{% else %}available{% endif %}
+**since version {{sch.since_version}}{% if
+sch.domain %} of domain {{sch.domain}}{% endif %}**.
+{% if len(sch.versions) > 1 %}
+Other versions of this operator:
+{% for v in sch.version[:-1] %} {{v}} {% endfor %}
+{% endif %}
+{% endif %}
+
+### Summary
+
+{{process_documentation(sch.doc)}}
+
+{% if sch.has_function %}
+
+#### Function Body
+
+The function definition for this operator.
+
+```
+{{get_function_body(sch)}}
+```
+{% endif %}
+{% if sch.attributes %}
+
+### Attributes
+
+{% for _, attr in sorted(sch.attributes.items())
+%}* **{{attr.name}} - {{str(attr.type).split('.')[-1]}}**{%
+  if attr.required %} (required){% endif %} {%
+  if attr.default_value %}{{clean_default_value(attr)}}{%
+  endif %}:
+
+{{text_indent(attr.description, 2)}}
+
+{% endfor %}
+{% endif %}
+{% if sch.inputs %}
+
+### Inputs
+
+{% if sch.min_input != sch.max_input %}Between {{sch.min_input
+}} and {{sch.max_input}} inputs.
+{% endif %}
+{% for ii, inp in enumerate(sch.inputs) %}
+- **{{getname(inp, ii)}}**{{format_option(inp)}} - **{{inp.type_str}}**:
+
+{{text_indent(inp.description, 2)}}{% endfor %}
+{% endif %}
+{% if sch.outputs %}
+
+### Outputs
+
+{% if sch.min_output != sch.max_output %}Between {{sch.min_output
+}} and {{sch.max_output}} outputs.
+{% endif %}
+{% for ii, out in enumerate(sch.outputs) %}
+- **{{getname(out, ii)}}**{{format_option(out)}} - **{{out.type_str}}**:
+
+{{text_indent(out.description, 2)}}{% endfor %}
+{% endif %}
+{% if sch.type_constraints %}
+
+### Type Constraints
+
+{% for ii, type_constraint in enumerate(sch.type_constraints)
+%}* {{get_constraint(type_constraint, ii)}}:
+
+{{text_indent(type_constraint.description, 2)}}
+{% endfor %}
+{% endif %}
+{% if examples and is_last_schema(sch): %}
+
+### Examples
+
+{% for example, code in examples.items(): %}
+
+#### {{ example }}
+
+```python
+{{ format_example(code) }}
+```
+{% endfor %}
+{% endif %}
+{% endfor %}""",
+    autoescape=False,
 )
 
 
-def get_template():  # type: ignore
-    try:
-        from jinja2 import Template
-    except ImportError:
-
-        class Template:  # type: ignore
-            "Docstring template"
-
-            def __init__(self, *args):
-                pass
-
-            def render(self, **context):
-                "render"
-                schemas = context["schemas"]
-                rows = []
-                for sch in schemas:
-                    doc = sch.doc or ""
-                    name = sch.name
-                    if name is None:
-                        raise RuntimeError("An operator must have a name.")
-                    rows.extend([name, "=" * len(name), "", doc, ""])
-                return "\n".join(rows)
-
-    return Template
-
-
-def _get_diff_template():  # type: ignore
-    Template = get_template()
-    return Template(
+def _get_main_template():
+    return jinja2.Template(
         textwrap.dedent(
             """
-        <div id="{{ div_name }}"></div>
-        <link rel="stylesheet" type="text/css" href="../_static/diff2html.min.css" />
-        <script type="text/javascript" src="../_static/diff2html-ui-slim.min.js"></script>
-        <script>
-        const diffString = `
-        --- a/{{ op_name }}{{ version1 }}
-        +++ b/{{ op_name }}{{ version2 }}
-        @@ -1 +1 @@
-        {{ diff_content }}
-        `;
+            .. _l-onnx-operators:
 
-        document.addEventListener('DOMContentLoaded', function () {
-        var targetElement = document.getElementById('{{ div_name }}');
-        var configuration = {
-            drawFileList: true,
-            fileListToggle: false,
-            fileListStartVisible: false,
-            fileContentToggle: false,
-            matching: 'lines',
-            outputFormat: 'line-by-line',
-            synchronisedScroll: true,
-            highlight: true,
-            renderNothingWhenEmpty: false,
-        };
-        var diff2htmlUi = new Diff2HtmlUI(targetElement, diffString, configuration);
-        diff2htmlUi.draw();
-        diff2htmlUi.highlightCode();
-        });
-        </script>
-        """
-        ),
-        autoescape=True,
-    )
+            {{ title }}
+            {{ "=" * len(title) }}
 
+            Lists out all the ONNX operators. For each operator, lists out the usage guide,
+            parameters, examples, and line-by-line version history.
+            This section also includes tables detailing each operator
+            with its versions, as done in `Operators.md
+            <https://github.com/onnx/onnx/blob/main/docs/Operators.md>`_.
 
-def _get_ops_template():  # type: ignore
-    Template = get_template()
-    return Template(
-        textwrap.dedent(
+            All examples end by calling function `expect`.
+            which checks a runtime produces the expected output for this example.
+            One implementation based on `onnxruntime <https://onnxruntime.ai/>`_
+            can be found at :ref:`l-function-expect`.
+
+            .. toctree::
+                :hidden:
+
+                ../expect_onnxruntime
+                {% for p in pages %}{{ os.path.split(p)[-1] }}
+                {% endfor %}
+
+            .. tabs::
+
+                {% for t in tabs %}.. tab:: {{ t.domain_name }}
+                    {{ t.render(indent="        ") }}
+                {% endfor %}
             """
-        {% for sch in schemas %}
-
-        .. tag-diff-insert.
-        .. _l-onnx-op{{sch.domain.lower().replace(".", "-")}}-{{sch.name.lower()}}-{{str(sch.since_version)}}:
-
-        {{format_name_with_domain(sch)}}
-        {{'=' * len(format_name_with_domain(sch))}}
-
-        **Version**
-
-        * **name**: `{{sch.name}} (GitHub) <{{build_doc_url(sch)}}{{sch.name}}>`_
-        * **domain**: **{% if sch.domain == '' %}main{% else %}{{sch.domain}}{% endif %}**
-        * **since_version**: **{{sch.since_version}}**
-        * **function**: {{sch.has_function or sch.has_context_dependent_function}}
-        * **support_level**: {{sch.support_level}}
-        * **shape inference**: {{sch.has_type_and_shape_inference_function}}
-
-        {% if sch.support_level == OpSchema.SupportType.EXPERIMENTAL %}
-        No versioning maintained for experimental ops.
-        {% else %}
-        This version of the operator has been {% if
-        sch.deprecated %}deprecated{% else %}available{% endif %}
-        **since version {{sch.since_version}}{% if
-        sch.domain %} of domain {{sch.domain}}{% endif %}**.
-        {% if len(sch.versions) > 1 %}
-        Other versions of this operator:
-        {% for v in sch.version[:-1] %} {{v}} {% endfor %}
-        {% endif %}
-        {% endif %}
-
-        **Summary**
-
-        {{process_documentation(sch.doc)}}
-        {% if sch.attributes %}
-
-        **Attributes**
-
-        {% for _, attr in sorted(sch.attributes.items())
-        %}* **{{attr.name}} - {{str(attr.type).split('.')[-1]}}**{%
-          if attr.required %} (required){% endif %} {%
-          if attr.default_value %}{{clean_default_value(attr)}}{%
-          endif %}: {{text_wrap(attr.description, 2)}}
-        {% endfor %}
-        {% endif %}
-        {% if sch.inputs %}
-
-        **Inputs**
-
-        {% if sch.min_input != sch.max_input %}Between {{sch.min_input
-        }} and {{sch.max_input}} inputs.
-        {% endif %}
-        {% for ii, inp in enumerate(sch.inputs) %}
-        * **{{getname(inp, ii)}}**{{format_option(inp)}} - **{{inp.type_str}}**:
-        {{text_wrap(inp.description, 2)}}{% endfor %}
-        {% endif %}
-        {% if sch.outputs %}
-
-        **Outputs**
-
-        {% if sch.min_output != sch.max_output %}Between {{sch.min_output
-        }} and {{sch.max_output}} outputs.
-        {% endif %}
-        {% for ii, out in enumerate(sch.outputs) %}
-        * **{{getname(out, ii)}}**{{format_option(out)}} - **{{out.type_str}}**:
-        {{text_wrap(out.description, 2)}}{% endfor %}
-        {% endif %}
-        {% if sch.type_constraints %}
-
-        **Type Constraints**
-
-        {% for ii, type_constraint in enumerate(sch.type_constraints)
-        %}* {{get_constraint(type_constraint, ii)}}:
-        {{text_wrap(type_constraint.description, 2)}}
-        {% endfor %}
-        {% endif %}
-        {% if get_onnx_example and is_last_schema(sch): %}
-
-        **Examples**
-
-        {% for example, code in get_onnx_example(sch.name).items(): %}
-
-        **{{ example }}**
-
-        ::
-
-        {{ format_example(code) }}
-        {% endfor %}
-        {% endif %}
-        {% endfor %}
-    """
-        ),
-        autoescape=True,
-    )
-
-
-def _get_main_template():  # type: ignore
-    Template = get_template()
-    return Template(
-        textwrap.dedent(
-            """
-        .. _l-onnx-operators:
-
-        {{ title }}
-        {{ "=" * len(title) }}
-
-        Lists out all the ONNX operators. For each operator, lists out the usage guide,
-        parameters, examples, and line-by-line version history.
-        This section also includes tables detailing each operator
-        with its versions, as done in `Operators.md
-        <https://github.com/onnx/onnx/blob/main/docs/Operators.md>`_.
-
-        All examples end by calling function `expect`.
-        which checks a runtime produces the expected output for this example.
-        One implementation based on `onnxruntime <https://onnxruntime.ai/>`_
-        can be found at :ref:`l-function-expect`.
-
-        .. toctree::
-            :hidden:
-
-            ../expect_onnxruntime
-            {% for p in pages %}{{ os.path.split(p)[-1] }}
-            {% endfor %}
-
-        .. tabs::
-
-            {% for t in tabs %}.. tab:: {{ t.domain_name }}
-                {{ t.render(indent="        ") }}
-            {% endfor %}
-    """
         ),
         autoescape=True,
     )
@@ -245,7 +238,7 @@ def _clean_unicode(text):
 _template_diff = _get_diff_template()
 _template_operator = _get_ops_template()
 _template_main = _get_main_template()
-__get_all_schemas_with_history = None
+_all_schemas_with_history = None
 
 
 _attribute_conversion_functions = {
@@ -259,16 +252,16 @@ _attribute_conversion_functions = {
     # AttributeProto.SPARSE_TENSORS(12)
     onnx.AttributeProto.STRING: lambda att: att.s.decode("utf-8"),
     onnx.AttributeProto.STRINGS: lambda att: [s.decode("utf-8") for s in att.strings],
-    onnx.AttributeProto.TENSOR: lambda att: to_array(att.t),
+    onnx.AttributeProto.TENSOR: lambda att: onnx.numpy_helper.to_array(att.t),
     # AttributeProto.TENSORS(9)
     # onnx.AttributeProto.TYPE_PROTO: lambda att: OnnxType(att.tp),
     # AttributeProto.TYPE_PROTOS(14)
 }
 
 
-def _populate__get_all_schemas_with_history():  # type: ignore
-    res = {}  # type: ignore
-    for schema in get_all_schemas_with_history():
+def _populate_all_schemas_with_history():
+    res: dict[str, Any] = {}
+    for schema in onnx.defs.get_all_schemas_with_history():
         domain = schema.domain
         version = schema.since_version
         name = schema.name
@@ -281,21 +274,14 @@ def _populate__get_all_schemas_with_history():  # type: ignore
     return res
 
 
-def _get_all_schemas_with_history():  # type: ignore
-    global __get_all_schemas_with_history  # pylint: disable=W0603
-    if __get_all_schemas_with_history is None:
-        __get_all_schemas_with_history = _populate__get_all_schemas_with_history()
-    return __get_all_schemas_with_history
+def _get_all_schemas_with_history():
+    global _all_schemas_with_history
+    if _all_schemas_with_history is None:
+        _all_schemas_with_history = _populate_all_schemas_with_history()
+    return _all_schemas_with_history
 
 
-def get_domain_list():  # type: ignore
-    """
-    Returns the list of available domains.
-    """
-    return list(sorted(set(map(lambda s: s.domain, get_all_schemas_with_history()))))
-
-
-def get_operator_schemas(op_name, version=None, domain=None):  # type: ignore
+def get_operator_schemas(op_name, version=None, domain=None):
     """
     Returns all schemas mapped to an operator name.
     :param op_name: name of the operator
@@ -305,7 +291,7 @@ def get_operator_schemas(op_name, version=None, domain=None):  # type: ignore
     """
     if version == "last" and op_name is not None:
         if domain is not None:
-            return [get_schema(op_name, domain=domain)]
+            return [onnx.defs.get_schema(op_name, domain=domain)]
     all_schemas = _get_all_schemas_with_history()
     if domain is None:
         domains = []
@@ -325,8 +311,8 @@ def get_operator_schemas(op_name, version=None, domain=None):  # type: ignore
                     sch.extend(v.values())
                 elif version == "last" and (dom == "" or "onnx" in dom):
                     try:
-                        sch.append(get_schema(op, domain=dom))
-                    except SchemaError:  # pragma: no cover
+                        sch.append(onnx.defs.get_schema(op, domain=dom))
+                    except onnx.defs.SchemaError:
                         sch.append(v[max(v)])
                 elif version == "last":
                     sch.append(v[max(v)])
@@ -344,7 +330,7 @@ def get_operator_schemas(op_name, version=None, domain=None):  # type: ignore
     return [v[-1] for v in vals]
 
 
-def get_rst_doc(  # type: ignore
+def get_markdown_doc(
     folder,
     op_name=None,
     domain=None,
@@ -354,7 +340,7 @@ def get_rst_doc(  # type: ignore
     example=False,
 ):
     """
-    Returns a documentation in RST format
+    Returns a documentation in Markdown format
     for all :class:`OnnxOperator`.
 
     :param op_name: operator name of None for all
@@ -364,15 +350,9 @@ def get_rst_doc(  # type: ignore
     :param diff: highlights differences between two versions
     :param example: add example to the documentation
     :return: string
-    The function relies on module `jinja2` or replaces it
-    with a simple rendering if not present.
     """
     schemas = get_operator_schemas(op_name, domain=domain, version=version)
 
-    # from onnx.backend.sample.ops import collect_sample_implementations
-    # from onnx.backend.test.case import collect_snippets
-    # SNIPPETS = collect_snippets()
-    # SAMPLE_IMPLEMENTATIONS = collect_sample_implementations()
     def format_name_with_domain(sch):
         if version == "last":
             if sch.domain:
@@ -395,7 +375,6 @@ def get_rst_doc(  # type: ignore
         return ""
 
     def format_example(code):
-        code = textwrap.indent(code, "    ")
         return code
 
     def get_constraint(const, ii):
@@ -405,7 +384,7 @@ def get_rst_doc(  # type: ignore
             name = str(ii)
         name = f"**{name}** in ("
         if const.allowed_type_strs:
-            types = [f"``{type_str}``" for type_str in sorted(const.allowed_type_strs)]
+            types = [f"`{type_str}`" for type_str in sorted(const.allowed_type_strs)]
             text = ", ".join(types)
             name += " " + text + " )"
         return name
@@ -420,41 +399,15 @@ def get_rst_doc(  # type: ignore
         if doc is None:
             doc = ""
         if not isinstance(doc, str):
-            raise TypeError(  # pragma: no cover
-                f"doc must be a string not {type(doc)!r} - {doc + 42!r}."
-            )
-        doc = textwrap.dedent(doc)
+            raise TypeError(f"doc must be a string not {type(doc)!r} - {doc + 42!r}.")
         main_docs_url = "https://github.com/onnx/onnx/blob/main/"
         rep = {
-            "[the doc](IR.md)": "`ONNX <{0}docs/IR.md>`_",
-            "[the doc](Broadcasting.md)": "`Broadcasting in ONNX <{0}docs/Broadcasting.md>`_",
-            "<dl>": "",
-            "</dl>": "",
-            "<dt>": "* ",
-            "<dd>": "  ",
-            "</dt>": "",
-            "</dd>": "",
-            "<tt>": "``",
-            "</tt>": "``",
-            "<br>": "\n",
+            "[the doc](IR.md)": f"[ONNX IR]({main_docs_url}docs/IR.md)",
+            "[the doc](Broadcasting.md)": f"[Broadcasting in ONNX]({main_docs_url}docs/Broadcasting.md)",
         }
-        for k, v in rep.items():
-            doc = doc.replace(k, v.format(main_docs_url))
-        move = 0
-        lines = []
-        for line in doc.split("\n"):
-            if line.startswith("```"):
-                if move > 0:
-                    move -= 4
-                    lines.append("\n")
-                else:
-                    lines.append("::\n")
-                    move += 4
-            elif move > 0:
-                lines.append(" " * move + line)
-            else:
-                lines.append(line)
-        return "\n".join(lines)
+        for key, value in rep.items():
+            doc = doc.replace(key, value)
+        return textwrap.dedent(doc)
 
     def build_doc_url(sch):
         doc_url = "https://github.com/onnx/onnx/blob/main/docs/Operators"
@@ -486,22 +439,23 @@ def get_rst_doc(  # type: ignore
         ):
             if attr.type in _attribute_conversion_functions:
                 sval = _attribute_conversion_functions[attr.type](default_value)
-                return f"(default is ``{sval!r}``)"
+                return f"(default is `{sval!r}`)"
 
         if isinstance(default_value, list):
             sval = [format_default_value(val) for val in default_value]
         else:
             sval = format_default_value(default_value)
-        return f"(default is ``{sval!r}``)"
+        return f"(default is `{sval!r}`)"
 
-    def text_wrap(text, indent):
+    def text_indent(text: str, indent: int) -> str:
         s = " " * indent
-        lines = textwrap.wrap(text, initial_indent=s, subsequent_indent=s)
-        return "\n".join(lines)
+        return textwrap.indent(text, s)
 
-    fnwd = format_name_with_domain
-    tmpl = _template_operator
-    docs = tmpl.render(
+    def get_function_body(schema: OpSchema) -> str:
+        return onnx.printer.to_text(schema.function_body)
+
+    examples = get_onnx_example(op_name, domain) if example else {}
+    docs = _template_operator.render(
         schemas=schemas,
         OpSchema=OpSchema,
         len=len,
@@ -511,17 +465,17 @@ def get_rst_doc(  # type: ignore
         get_constraint=get_constraint,
         getname=getname,
         enumerate=enumerate,
-        format_name_with_domain=fnwd,
+        format_name_with_domain=format_name_with_domain,
         process_documentation=process_documentation,
         build_doc_url=build_doc_url,
-        text_wrap=text_wrap,
+        text_indent=text_indent,
         str=str,
         clean_default_value=clean_default_value,
-        get_onnx_example=get_onnx_example if example else None,
+        examples=examples,
         format_example=format_example,
         is_last_schema=is_last_schema,
+        get_function_body=get_function_body,
     )
-    docs = _clean_unicode(docs)
 
     d_links = {}
     for schema in schemas:
@@ -559,44 +513,46 @@ def get_rst_doc(  # type: ignore
             new_lines.append(line)
         docs = "\n".join(new_lines)
 
-    return docs, d_links
+    return docs, d_links, len(examples)
 
 
-def _insert_diff(folder, docs, split=".. tag-diff-insert.", op_name=None, version=None, domain=None):  # type: ignore
+def _insert_diff(
+    folder, docs, split=".. tag-diff-insert.", op_name=None, version=None, domain=None
+):
     """
     Splits a using `split`, insert HTML differences between pieces.
     The function relies on package `pyquickhelper`.
     """
-    spl = docs.split(split)
-    if len(spl) <= 1:
+    doc_parts = docs.split(split)
+    if len(doc_parts) <= 1:
         return docs
 
     reg = re.compile("([A-Z][A-Za-z0-9_]*) - ([0-9]+)")
 
-    d_links = {}  # type: ignore
-    pieces = [spl[0]]  # type: ignore
-    mds = []  # type: ignore
-    for i in range(1, len(spl)):
-        spl1 = spl[i - 1].strip("\n ")
-        spl2 = spl[i].strip("\n ")
+    d_links = {}
+    pieces = [doc_parts[0]]
+    mds = []
+    for i in range(1, len(doc_parts)):
+        spl1 = doc_parts[i - 1].strip("\n ")
+        spl2 = doc_parts[i].strip("\n ")
         vers1 = reg.findall(spl1)
         vers2 = reg.findall(spl2)
 
-        spl1 = spl1.split("**Examples**")[0].replace("`", "")
-        spl2 = spl2.split("**Examples**")[0].replace("`", "")
-        spl1 = spl1.split("**Summary**")[-1].strip("\n ")
-        spl2 = spl2.split("**Summary**")[-1].strip("\n ")
+        spl1 = spl1.split("### Examples")[0].replace("`", "")
+        spl2 = spl2.split("### Examples")[0].replace("`", "")
+        spl1 = spl1.split("### Summary")[-1].strip("\n ")
+        spl2 = spl2.split("### Summary")[-1].strip("\n ")
         if len(spl1) < 5 or len(spl2) < 5:
-            pieces.append(spl[i])
+            pieces.append(doc_parts[i])
             continue
-        if len(vers1) == 0:
+        if not vers1:
             raise ValueError(f"Unable to find version {version!r} in\n{spl1}")
-        if len(vers2) == 0:
+        if not vers2:
             raise ValueError(f"Unable to find version {version!r} in\n{spl2}")
         v2 = vers2[0][1]
         v1 = vers1[0][1]
 
-        if len(mds) == 0:
+        if not mds:
             mds.append(
                 (v1, textwrap.dedent(spl1.strip(" \n\r\t")).splitlines(keepends=True))
             )
@@ -605,19 +561,23 @@ def _insert_diff(folder, docs, split=".. tag-diff-insert.", op_name=None, versio
         )
 
         if len(mds) > 1:
-            pieces.extend([".. toctree::", ""])
+            show_diff_toc = True
+        else:
+            show_diff_toc = False
+
+        if show_diff_toc:
+            pieces.append("```{toctree}")
 
         for di in range(len(mds) - 1):
             dj = len(mds) - 1
 
             v1, s1 = mds[di]
             v2, s2 = mds[dj]
-            d = Differ()
-            result = list(d.compare(s2, s1))
+            differ = difflib.Differ()
+            result = list(differ.compare(s2, s1))
             raw = "".join(result)
 
-            tmpl = _template_diff
-            diff = tmpl.render(
+            diff = _template_diff.render(
                 op_name=op_name,
                 version1=v2,
                 version2=v1,
@@ -629,8 +589,8 @@ def _insert_diff(folder, docs, split=".. tag-diff-insert.", op_name=None, versio
             title = f"{op_name} - {v2} vs {v1}"
 
             name = f"text_diff_{op_name}_{v2}_{v1}"
-            sdom = domain.replace(".", "-")
-            link = f"l-onnx-op{sdom}-{op_name.lower()}-d{v2}-{v1}"
+            domain_str = domain.replace(".", "-")
+            link = f"l-onnx-op{domain_str}-{op_name.lower()}-d{v2}-{v1}"
             d_links[int(v2), int(v1)] = link
             content = "\n".join(
                 [
@@ -651,23 +611,20 @@ def _insert_diff(folder, docs, split=".. tag-diff-insert.", op_name=None, versio
                 ]
             )
             filename = os.path.join(folder, name + ".rst")
-            if os.path.exists(filename):
-                with open(filename, "r", encoding="utf-8") as f:
-                    old_content = f.read()
-                    write = old_content != content
-            else:
-                write = True
-            if write:
-                with open(filename, "w", encoding="utf-8") as f:
-                    f.write(content)
-            pieces.append(f"    {name}")
+            pathlib.Path(filename).write_text(content, encoding="utf-8")
+            # Add diff page to the toctree using myst syntax
+            pieces.append(name)
 
-        pieces.extend(["", spl[i]])
+        if show_diff_toc:
+            # End the toctree
+            pieces.append("```")
+
+        pieces.extend(["", doc_parts[i]])
 
     return "\n".join(pieces), d_links
 
 
-def change_style(name: str) -> str:
+def pascal_to_snake_case(name: str) -> str:
     """
     Switches from *AaBb* into *aa_bb*.
     :param name: name to convert
@@ -682,24 +639,32 @@ def _process_example(code: str) -> str:
     """
     Add necessary imports to make the example work.
     """
-    code = code.replace("  # type: ignore", "")
+    code = code.replace("", "")
     missing_imports = ["import numpy as np", "import onnx"]
-    elements = missing_imports + ["", "", code.strip("\n"), ""]
+    elements = [*missing_imports, "", "", code.strip("\n")]
     return "\n".join(elements)
 
 
-def get_onnx_example(op_name):  # type: ignore
+def get_onnx_example(op_name, domain):
     """
     Retrieves examples associated to one operator
     stored in onnx packages.
     :param op_name: operator name
+    :param domain: operator domain
     :param fmt: rendering format
     :return: dictionary
     """
-    modules = [
-        f"onnx.backend.test.case.node.{op_name.lower()}",
-        f"onnx.backend.test.case.node.{change_style(op_name).lower()}",
-    ]
+    if domain in (None, "ai.onnx"):
+        modules = [
+            f"onnx.backend.test.case.node.{op_name.lower()}",
+            f"onnx.backend.test.case.node.{pascal_to_snake_case(op_name)}",
+        ]
+    else:
+        domain_ = domain.replace(".", "_")
+        modules = [
+            f"onnx.backend.test.case.node.{domain_}.{op_name.lower()}",
+            f"onnx.backend.test.case.node.{domain_}.{pascal_to_snake_case(op_name)}",
+        ]
     module = None
     for m in modules:
         try:
@@ -710,7 +675,7 @@ def get_onnx_example(op_name):  # type: ignore
     if module is None:
         # Unable to find an example for 'op_name'.
         return {}
-    results = {}  # type: ignore
+    results: dict[str, Any] = {}
     for v in mod.__dict__.values():
         if not isinstance(v, _Exporter):
             continue
@@ -725,13 +690,11 @@ def get_onnx_example(op_name):  # type: ignore
                 if sub in code:
                     found = code
             if found is None:
-                raise RuntimeError(  # pragma: no cover
-                    f"Unable to find {sub!r} in\n{code_cls}"
-                )
+                raise RuntimeError(f"Unable to find {sub!r} in\n{code_cls}")
             found = textwrap.dedent(found)
             lines = found.split("\n")
             first = 0
-            for i in range(len(lines)):  # pylint: disable=C0200
+            for i in range(len(lines)):
                 if lines[i].startswith("def "):
                     first = i + 1
             found = textwrap.dedent("\n".join(lines[first:]))
@@ -751,16 +714,15 @@ def is_last_schema(sch: OpSchema) -> bool:
     :return: True
     """
     try:
-        last = get_schema(sch.name, domain=sch.domain)
-    except SchemaError:  # pragma: no cover
-        # raise RuntimeError(
-        #     "Unable to find schema for operator %r and domain %r."
-        #     "" % (sch.name, sch.domain))
+        last = onnx.defs.get_schema(sch.name, domain=sch.domain)
+    except onnx.defs.SchemaError:
         return True
     return last.since_version == sch.since_version
 
 
-def onnx_documentation_folder(folder, title="ONNX Operators", flog=None, max_opsets=None):  # type: ignore
+def onnx_documentation_folder(
+    folder, title="ONNX Operators", flog=None, max_opsets=None
+):
     """
     Creates documentation in a folder for all known
     ONNX operators or a subset.
@@ -801,30 +763,22 @@ def onnx_documentation_folder(folder, title="ONNX Operators", flog=None, max_ops
             for op in self.ops:
                 name = op["name"]
                 dom = self.domain.replace(".", "-")
-                table_dom.append(f"    * - :ref:`l-onnx-doc{dom}-{name}`")
-                versions = list(
-                    reversed(
-                        sorted(
-                            (k, v) for k, v in op["links"].items() if isinstance(k, int)
-                        )
-                    )
+                table_dom.append(f"    * - :ref:`{name} <l-onnx-doc{dom}-{name}>`")
+                versions = sorted(
+                    [(k, v) for k, v in op["links"].items() if isinstance(k, int)],
+                    reverse=True,
                 )
                 col1 = ", ".join(f":ref:`{k} <{v}>`" for k, v in versions)
-                diffs = list(
-                    reversed(
-                        sorted(
-                            (k, v)
-                            for k, v in op["links"].items()
-                            if isinstance(k, tuple)
-                        )
-                    )
+                diffs = sorted(
+                    [(k, v) for k, v in op["links"].items() if isinstance(k, tuple)],
+                    reverse=True,
                 )
                 col2 = ", ".join(f":ref:`{k[1]}/{k[0]} <{v}>`" for k, v in diffs)
                 table_dom.append(f"      - {col1}")
                 table_dom.append(f"      - {col2}")
             table_dom.append("")
             if indent != "":
-                for i in range(len(table_dom)):  # pylint: disable=C0200
+                for i in range(len(table_dom)):
                     table_dom[i] = indent + table_dom[i]
             res = "\n".join(table_dom)
             return res
@@ -872,39 +826,32 @@ def onnx_documentation_folder(folder, title="ONNX Operators", flog=None, max_ops
         # loop on operators
         for op in sorted(do):
             if flog is not None:
-                flog(f"generate page for onnx {dom!r} - {op!r}")  # pragma: no cover
+                flog(f"generate page for onnx {dom!r} - {op!r}")
             page_name = f"onnx_{dom.replace('.', '')}_{op}"
-            doc, d_links = get_rst_doc(
+            doc, d_links, n_examples = get_markdown_doc(
                 folder, op, domain=dom, version=None, example=True, diff=True
             )
+            if flog is not None and n_examples == 0:
+                flog(f"{' '* 14}no_example for {op} from domain {domain}")
             if dom == "":
                 main = op
             else:
                 main = f"{dom} - {op}"
             sdom = dom.replace(".", "-")
-            ref_link = f".. _l-onnx-doc{sdom}-{op}:"
+            # Target in MyST https://myst-parser.readthedocs.io/en/v0.15.1/syntax/syntax.html?highlight=role#extra-markdown-syntax
+            ref_link = f"(l-onnx-doc{sdom}-{op})="
             rows = [
                 "",
                 ref_link,
                 "",
-                "=" * len(main),
-                main,
-                "=" * len(main),
+                f"# {main}",
                 "",
                 doc,
             ]
 
-            full = os.path.join(folder, page_name + ".rst")
+            full = os.path.join(folder, page_name + ".md")
             content = "\n".join(rows)
-            if os.path.exists(full):
-                with open(full, "r", encoding="utf-8") as f:
-                    old_content = f.read()
-                write = old_content != content
-            else:
-                write = True
-            if write:
-                with open(full, "w", encoding="utf-8") as f:
-                    f.write(content)
+            pathlib.Path(full).write_text(content, encoding="utf-8")
             pages.append(full)
             dom_pages.append({"name": op, "links": d_links})
 
@@ -913,12 +860,10 @@ def onnx_documentation_folder(folder, title="ONNX Operators", flog=None, max_ops
     # final
     if len(tables) < 3:
         raise RuntimeError(f"At least three domain are expected not {len(tables)}.")
-    tmpl = _template_main
-    index = tmpl.render(pages=pages, tabs=tables, os=os, len=len, title=title)
+    index = _template_main.render(pages=pages, tabs=tables, os=os, len=len, title=title)
     index = _clean_unicode(index)
     page_name = os.path.join(folder, "index.rst")
-    with open(page_name, "w", encoding="utf-8") as f:
-        f.write(index)
+    pathlib.Path(page_name).write_text(index, encoding="utf-8")
     pages.append(page_name)
     return pages
 
@@ -930,6 +875,23 @@ def _generate_op_doc(app):
     onnx_documentation_folder(folder, flog=logger.info, max_opsets=max_opsets)
 
 
+def _copy_repo_docs(app):
+    logger = logging.getLogger(__name__)
+    dest_name = app.config.onnx_md_folder
+
+    docs_dir = pathlib.Path(__file__).parent.parent.parent  # docs
+    dest_folder = docs_dir / "docsgen" / "source" / dest_name
+    dest_folder.mkdir(parents=True, exist_ok=True)
+    # Copy all the markdown files from the folder except for the blocklisted ones
+
+    logger.info("Copying Markdown files from '%s' to '%s'", docs_dir, dest_folder)
+    for file in docs_dir.glob("*.md"):
+        if file.name in REPO_DOCS_EXCLUDE:
+            continue
+        shutil.copy(file, dest_folder)
+        logger.info("Copying '%s'", file.name)
+
+
 def setup(app):
     """
     Sphinx extension `onnx_sphinx` displays documentation
@@ -938,8 +900,11 @@ def setup(app):
     import sphinx
 
     app.add_config_value("onnx_doc_folder", "operators", "env")
+    # Folder for storing the Markdown documentation from the repository
+    app.add_config_value("onnx_md_folder", "repo-docs", "env")
     app.add_config_value("max_opsets", {}, "env")
     app.connect("builder-inited", _generate_op_doc)
+    app.connect("builder-inited", _copy_repo_docs)
     return {"version": sphinx.__display_version__, "parallel_read_safe": True}
 
 
