@@ -1,14 +1,16 @@
 # Copyright (c) ONNX Project Contributors
 #
 # SPDX-License-Identifier: Apache-2.0
+from __future__ import annotations
 
 import numpy as np
 
 import onnx
 from onnx.backend.test.case.base import Base
 from onnx.backend.test.case.node import expect
-from onnx.backend.test.case.node.pool_op_common import (
-    get_output_shape,
+from onnx.reference.ops.op_pool_common import (
+    get_output_shape_auto_pad,
+    get_output_shape_explicit_padding,
     get_pad_shape,
     pool,
 )
@@ -17,8 +19,7 @@ from onnx.backend.test.case.node.pool_op_common import (
 class AveragePool(Base):
     @staticmethod
     def export_averagepool_2d_precomputed_pads() -> None:
-        """
-        input_shape: [1, 1, 5, 5]
+        """input_shape: [1, 1, 5, 5]
         output_shape: [1, 1, 5, 5]
         pad_shape: [4, 4] -> [2, 2, 2, 2] by axis
         """
@@ -62,8 +63,7 @@ class AveragePool(Base):
 
     @staticmethod
     def export_averagepool_2d_precomputed_pads_count_include_pad() -> None:
-        """
-        input_shape: [1, 1, 5, 5]
+        """input_shape: [1, 1, 5, 5]
         output_shape: [1, 1, 5, 5]
         pad_shape: [4, 4] -> [2, 2, 2, 2] by axis
         """
@@ -111,8 +111,7 @@ class AveragePool(Base):
 
     @staticmethod
     def export_averagepool_2d_precomputed_strides() -> None:
-        """
-        input_shape: [1, 1, 5, 5]
+        """input_shape: [1, 1, 5, 5]
         output_shape: [1, 1, 2, 2]
         """
         node = onnx.helper.make_node(
@@ -146,8 +145,7 @@ class AveragePool(Base):
 
     @staticmethod
     def export_averagepool_2d_precomputed_same_upper() -> None:
-        """
-        input_shape: [1, 1, 5, 5]
+        """input_shape: [1, 1, 5, 5]
         output_shape: [1, 1, 3, 3]
         pad_shape: [2, 2] -> [1, 1, 1, 1] by axis
         """
@@ -185,8 +183,7 @@ class AveragePool(Base):
 
     @staticmethod
     def export_averagepool_1d_default() -> None:
-        """
-        input_shape: [1, 3, 32]
+        """input_shape: [1, 3, 32]
         output_shape: [1, 3, 31]
         """
         node = onnx.helper.make_node(
@@ -197,18 +194,20 @@ class AveragePool(Base):
         )
         x = np.random.randn(1, 3, 32).astype(np.float32)
         x_shape = np.shape(x)
+        pads = None
         kernel_shape = [2]
         strides = [1]
-        out_shape = get_output_shape("VALID", x_shape[2:], kernel_shape, strides)
+        out_shape, _ = get_output_shape_explicit_padding(
+            pads, x_shape[2:], kernel_shape, strides
+        )
         padded = x
-        y = pool(padded, x_shape, kernel_shape, strides, out_shape, [0], "AVG")
+        y = pool(padded, x_shape, kernel_shape, strides, out_shape, "AVG")
 
         expect(node, inputs=[x], outputs=[y], name="test_averagepool_1d_default")
 
     @staticmethod
     def export_averagepool_2d_default() -> None:
-        """
-        input_shape: [1, 3, 32, 32]
+        """input_shape: [1, 3, 32, 32]
         output_shape: [1, 3, 31, 31]
         """
         node = onnx.helper.make_node(
@@ -219,18 +218,20 @@ class AveragePool(Base):
         )
         x = np.random.randn(1, 3, 32, 32).astype(np.float32)
         x_shape = np.shape(x)
+        pads = None
         kernel_shape = (2, 2)
         strides = (1, 1)
-        out_shape = get_output_shape("VALID", x_shape[2:], kernel_shape, strides)
+        out_shape, _ = get_output_shape_explicit_padding(
+            pads, x_shape[2:], kernel_shape, strides
+        )
         padded = x
-        y = pool(padded, x_shape, kernel_shape, strides, out_shape, (0, 0), "AVG")
+        y = pool(padded, x_shape, kernel_shape, strides, out_shape, "AVG")
 
         expect(node, inputs=[x], outputs=[y], name="test_averagepool_2d_default")
 
     @staticmethod
     def export_averagepool_3d_default() -> None:
-        """
-        input_shape: [1, 3, 32, 32, 32]
+        """input_shape: [1, 3, 32, 32, 32]
         output_shape: [1, 3, 31, 31, 31]
         """
         node = onnx.helper.make_node(
@@ -241,18 +242,20 @@ class AveragePool(Base):
         )
         x = np.random.randn(1, 3, 32, 32, 32).astype(np.float32)
         x_shape = np.shape(x)
+        pads = None
         kernel_shape = [2, 2, 2]
         strides = [1, 1, 1]
-        out_shape = get_output_shape("VALID", x_shape[2:], kernel_shape, strides)
+        out_shape, _ = get_output_shape_explicit_padding(
+            pads, x_shape[2:], kernel_shape, strides
+        )
         padded = x
-        y = pool(padded, x_shape, kernel_shape, strides, out_shape, [0, 0, 0], "AVG")
+        y = pool(padded, x_shape, kernel_shape, strides, out_shape, "AVG")
 
         expect(node, inputs=[x], outputs=[y], name="test_averagepool_3d_default")
 
     @staticmethod
     def export_averagepool_2d_same_upper() -> None:
-        """
-        input_shape: [1, 3, 32, 32]
+        """input_shape: [1, 3, 32, 32]
         output_shape: [1, 3, 32, 32]
         pad_shape: [1, 1] -> [0, 1, 0, 1] by axis
         """
@@ -267,7 +270,9 @@ class AveragePool(Base):
         x_shape = np.shape(x)
         kernel_shape = (2, 2)
         strides = (1, 1)
-        out_shape = get_output_shape("SAME_UPPER", x_shape[2:], kernel_shape, strides)
+        out_shape = get_output_shape_auto_pad(
+            "SAME_UPPER", x_shape[2:], kernel_shape, strides
+        )
         pad_shape = get_pad_shape(
             "SAME_UPPER", x_shape[2:], kernel_shape, strides, out_shape
         )
@@ -281,14 +286,14 @@ class AveragePool(Base):
             mode="constant",
             constant_values=np.nan,
         )
-        y = pool(padded, x_shape, kernel_shape, strides, out_shape, pad_shape, "AVG")
+        pads = (pad_top, pad_left, pad_bottom, pad_right)
+        y = pool(padded, x_shape, kernel_shape, strides, out_shape, "AVG", pads)
 
         expect(node, inputs=[x], outputs=[y], name="test_averagepool_2d_same_upper")
 
     @staticmethod
     def export_averagepool_2d_same_lower() -> None:
-        """
-        input_shape: [1, 3, 32, 32]
+        """input_shape: [1, 3, 32, 32]
         output_shape: [1, 3, 32, 32]
         pad_shape: [1, 1] -> [1, 0, 1, 0] by axis
         """
@@ -303,7 +308,9 @@ class AveragePool(Base):
         x_shape = np.shape(x)
         kernel_shape = (2, 2)
         strides = (1, 1)
-        out_shape = get_output_shape("SAME_LOWER", x_shape[2:], kernel_shape, strides)
+        out_shape = get_output_shape_auto_pad(
+            "SAME_LOWER", x_shape[2:], kernel_shape, strides
+        )
         pad_shape = get_pad_shape(
             "SAME_LOWER", x_shape[2:], kernel_shape, strides, out_shape
         )
@@ -317,14 +324,14 @@ class AveragePool(Base):
             mode="constant",
             constant_values=np.nan,
         )
-        y = pool(padded, x_shape, kernel_shape, strides, out_shape, pad_shape, "AVG")
+        pads = (pad_top, pad_left, pad_bottom, pad_right)
+        y = pool(padded, x_shape, kernel_shape, strides, out_shape, "AVG", pads)
 
         expect(node, inputs=[x], outputs=[y], name="test_averagepool_2d_same_lower")
 
     @staticmethod
     def export_averagepool_2d_pads() -> None:
-        """
-        input_shape: [1, 3, 28, 28]
+        """input_shape: [1, 3, 28, 28]
         output_shape: [1, 3, 30, 30]
         pad_shape: [4, 4] -> [2, 2, 2, 2] by axis
         """
@@ -343,24 +350,23 @@ class AveragePool(Base):
         pad_top = 2
         pad_right = 2
         pad_left = 2
-        pad_shape = [pad_top + pad_bottom, pad_left + pad_right]
-        out_shape = get_output_shape(
-            "VALID", np.add(x_shape[2:], pad_shape), kernel_shape, strides
+        pads = [pad_top, pad_left, pad_bottom, pad_right]
+        out_shape, pads = get_output_shape_explicit_padding(
+            pads, x_shape[2:], kernel_shape, strides, ceil_mode=False
         )
         padded = np.pad(
             x,
-            ((0, 0), (0, 0), (pad_top, pad_bottom), (pad_left, pad_right)),
+            ((0, 0), (0, 0), (pads[0], pads[2]), (pads[1], pads[3])),
             mode="constant",
             constant_values=np.nan,
         )
-        y = pool(padded, x_shape, kernel_shape, strides, out_shape, pad_shape, "AVG")
+        y = pool(padded, x_shape, kernel_shape, strides, out_shape, "AVG", pads)
 
         expect(node, inputs=[x], outputs=[y], name="test_averagepool_2d_pads")
 
     @staticmethod
     def export_averagepool_2d_pads_count_include_pad() -> None:
-        """
-        input_shape: [1, 3, 28, 28]
+        """input_shape: [1, 3, 28, 28]
         output_shape: [1, 3, 30, 30]
         pad_shape: [4, 4] -> [2, 2, 2, 2] by axis
         """
@@ -374,19 +380,20 @@ class AveragePool(Base):
         )
         x = np.random.randn(1, 3, 28, 28).astype(np.float32)
         x_shape = np.shape(x)
+        dilations = (1, 1)
         kernel_shape = (3, 3)
         strides = (1, 1)
         pad_bottom = 2
         pad_top = 2
         pad_right = 2
         pad_left = 2
-        pad_shape = [pad_top + pad_bottom, pad_left + pad_right]
-        out_shape = get_output_shape(
-            "VALID", np.add(x_shape[2:], pad_shape), kernel_shape, strides
+        pads = [pad_top, pad_left, pad_bottom, pad_right]
+        out_shape, pads = get_output_shape_explicit_padding(
+            pads, x_shape[2:], kernel_shape, strides, dilations, ceil_mode=False
         )
         padded = np.pad(
             x,
-            ((0, 0), (0, 0), (pad_top, pad_bottom), (pad_left, pad_right)),
+            ((0, 0), (0, 0), (pads[0], pads[2]), (pads[1], pads[3])),
             mode="constant",
             constant_values=0,
         )
@@ -396,8 +403,8 @@ class AveragePool(Base):
             kernel_shape,
             strides,
             out_shape,
-            pad_shape,
             "AVG",
+            pads,
             count_include_pad=1,
         )
 
@@ -410,8 +417,7 @@ class AveragePool(Base):
 
     @staticmethod
     def export_averagepool_2d_strides() -> None:
-        """
-        input_shape: [1, 3, 32, 32]
+        """input_shape: [1, 3, 32, 32]
         output_shape: [1, 3, 10, 10]
         """
         node = onnx.helper.make_node(
@@ -425,16 +431,17 @@ class AveragePool(Base):
         x_shape = np.shape(x)
         kernel_shape = (5, 5)
         strides = (3, 3)
-        out_shape = get_output_shape("VALID", x_shape[2:], kernel_shape, strides)
+        out_shape, pads = get_output_shape_explicit_padding(
+            None, x_shape[2:], kernel_shape, strides, ceil_mode=False
+        )
         padded = x
-        y = pool(padded, x_shape, kernel_shape, strides, out_shape, (0, 0), "AVG")
+        y = pool(padded, x_shape, kernel_shape, strides, out_shape, "AVG", pads)
 
         expect(node, inputs=[x], outputs=[y], name="test_averagepool_2d_strides")
 
     @staticmethod
     def export_averagepool_2d_ceil() -> None:
-        """
-        input_shape: [1, 1, 4, 4]
+        """input_shape: [1, 1, 4, 4]
         output_shape: [1, 1, 2, 2]
         """
         node = onnx.helper.make_node(
@@ -463,8 +470,7 @@ class AveragePool(Base):
 
     @staticmethod
     def export_averagepool_2d_dilations() -> None:
-        """
-        input_shape: [1, 1, 4, 4]
+        """input_shape: [1, 1, 4, 4]
         output_shape: [1, 1, 2, 2]
         """
         node = onnx.helper.make_node(
@@ -494,3 +500,115 @@ class AveragePool(Base):
         y = np.array([[[[6, 7], [10, 11]]]]).astype(np.float32)
 
         expect(node, inputs=[x], outputs=[y], name="test_averagepool_2d_dilations")
+
+    @staticmethod
+    def export_averagepool_3d_dilations() -> None:
+        """input_shape: [1, 1, 4, 4]
+        output_shape: [1, 1, 2, 2]
+        """
+        node = onnx.helper.make_node(
+            "AveragePool",
+            inputs=["x"],
+            outputs=["y"],
+            kernel_shape=[2, 2, 2],
+            strides=[1, 1, 1],
+            dilations=[2, 2, 2],
+            ceil_mode=True,
+        )
+
+        # input shape: [1, 1, 4, 4, 4]
+        x = np.array(
+            [
+                [
+                    [
+                        [
+                            [1, 2, 3, 4],
+                            [5, 6, 7, 8],
+                            [9, 10, 11, 12],
+                            [13, 14, 15, 16],
+                        ],
+                        [
+                            [1, 2, 3, 4],
+                            [5, 6, 7, 8],
+                            [9, 10, 11, 12],
+                            [13, 14, 15, 16],
+                        ],
+                        [
+                            [1, 2, 3, 4],
+                            [5, 6, 7, 8],
+                            [9, 10, 11, 12],
+                            [13, 14, 15, 16],
+                        ],
+                        [
+                            [1, 2, 3, 4],
+                            [5, 6, 7, 8],
+                            [9, 10, 11, 12],
+                            [13, 14, 15, 16],
+                        ],
+                    ]
+                ]
+            ]
+        ).astype(np.float32)
+
+        y = np.array([[[[[6, 7], [10, 11]], [[6, 7], [10, 11]]]]]).astype(np.float32)
+
+        expect(
+            node, inputs=[x], outputs=[y], name="test_averagepool_3d_dilations_small"
+        )
+
+    @staticmethod
+    def export_averagepool_3d_dilations_large() -> None:
+        x_shape = (32, 32, 32)
+        dilations = (2, 2, 2)
+        kernel_shape = (5, 5, 5)
+        strides = (3, 3, 3)
+        count_include_pad = 0
+
+        for count_include_pad in (0, 1):
+            for ceil_mode in (True, False):
+                node = onnx.helper.make_node(
+                    "AveragePool",
+                    inputs=["x"],
+                    outputs=["y"],
+                    kernel_shape=kernel_shape,
+                    strides=strides,
+                    dilations=dilations,
+                    count_include_pad=count_include_pad,
+                    ceil_mode=ceil_mode,
+                )
+
+                x = np.random.randn(1, 1, *x_shape).astype(np.float32)
+                out_shape, pads = get_output_shape_explicit_padding(
+                    None,
+                    x_shape,
+                    kernel_shape,
+                    strides,
+                    dilations=dilations,
+                    ceil_mode=ceil_mode,
+                )
+                padded = np.pad(
+                    x,
+                    (
+                        (0, 0),
+                        (0, 0),
+                        (pads[0], pads[3]),
+                        (pads[1], pads[4]),
+                        (pads[2], pads[5]),
+                    ),
+                    mode="constant",
+                    constant_values=0 if count_include_pad == 1 else np.nan,
+                )
+                y = pool(
+                    padded,
+                    (1, 1, *x_shape),
+                    kernel_shape,
+                    strides,
+                    out_shape,
+                    "AVG",
+                    pads=pads,
+                    dilations=dilations,
+                    count_include_pad=count_include_pad,
+                )
+
+                test_name = f"test_averagepool_3d_dilations_large_count_include_pad_is_{count_include_pad}_ceil_mode_is_{ceil_mode}"
+                expect(node, inputs=[x], outputs=[y], name=test_name)
