@@ -1,8 +1,9 @@
 # Copyright (c) ONNX Project Contributors
 #
 # SPDX-License-Identifier: Apache-2.0
+from __future__ import annotations
 
-from typing import Any, Tuple
+from typing import Any
 
 import numpy as np
 
@@ -37,7 +38,7 @@ class GRUHelper:
             hidden_size = params[R].shape[-1]
             batch_size = params[X].shape[1]
 
-            layout = params[LAYOUT] if LAYOUT in params else 0
+            layout = params.get(LAYOUT, 0)
             x = params[X]
             x = x if layout == 0 else np.swapaxes(x, 0, 1)
             b = (
@@ -46,7 +47,7 @@ class GRUHelper:
                 else np.zeros(2 * number_of_gates * hidden_size)
             )
             h_0 = params[H_0] if H_0 in params else np.zeros((batch_size, hidden_size))
-            lbr = params[LBR] if LBR in params else 0
+            lbr = params.get(LBR, 0)
 
             self.X = x
             self.W = params[W]
@@ -65,7 +66,7 @@ class GRUHelper:
     def g(self, x: np.ndarray) -> np.ndarray:
         return np.tanh(x)
 
-    def step(self) -> Tuple[np.ndarray, np.ndarray]:
+    def step(self) -> tuple[np.ndarray, np.ndarray]:
         seq_length = self.X.shape[0]
         hidden_size = self.H_0.shape[-1]
         batch_size = self.X.shape[1]
