@@ -1,11 +1,11 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
+from __future__ import annotations
 
 import importlib
 import pkgutil
 from types import ModuleType
-from typing import List, Optional
 
 import numpy as np
 
@@ -27,10 +27,8 @@ all_numeric_dtypes = [
 
 
 def import_recursive(package: ModuleType) -> None:
-    """
-    Takes a package and imports all modules underneath it
-    """
-    pkg_dir: Optional[List[str]] = None
+    """Takes a package and imports all modules underneath it."""
+    pkg_dir: list[str] | None = None
     pkg_dir = package.__path__  # type: ignore
     module_location = package.__name__
     for _module_loader, name, ispkg in pkgutil.iter_modules(pkg_dir):
