@@ -3,9 +3,10 @@
 # Copyright (c) ONNX Project Contributors
 #
 # SPDX-License-Identifier: Apache-2.0
+from __future__ import annotations
 
 import os
-from typing import IO, Any, Dict, List, Sequence
+from typing import IO, Any, Sequence
 
 from onnx import AttributeProto, defs, load
 from onnx.backend.test.case import collect_snippets
@@ -36,8 +37,8 @@ experimental_covered: Sequence[str] = []
 def gen_node_test_coverage(
     schemas: Sequence[defs.OpSchema], f: IO[Any], ml: bool
 ) -> None:
-    global common_covered
-    global experimental_covered
+    global common_covered  # noqa: PLW0603
+    global experimental_covered  # noqa: PLW0603
     generators = set(
         {
             "Multinomial",
@@ -98,7 +99,7 @@ def gen_node_test_coverage(
         f.write("Node tests have covered 0/0 (N/A) common operators. \n\n")
     if num_experimental:
         f.write(
-            "Node tests have covered {}/{} ({:.2f}%, {} generators excluded) "
+            "Node tests have covered {}/{} ({:.2f}%, {} generators excluded) "  # noqa: UP032
             "experimental operators.\n\n".format(
                 len(experimental_covered),
                 num_experimental,
@@ -136,7 +137,7 @@ def gen_node_test_coverage(
                     f.write(f"<summary>{summary}</summary>\n\n")
                     f.write(f"```python\n{code}\n```\n\n")
                     f.write("</details>\n")
-            else:
+            else:  # noqa: PLR5501
                 if s in generators:
                     f.write(" (random generator operator)\n")
                 else:
@@ -150,13 +151,13 @@ def gen_model_test_coverage(
 ) -> None:
     f.write("# Model Test Coverage\n")
     # Process schemas
-    schema_dict = dict()
+    schema_dict = {}
     for schema in schemas:
         schema_dict[schema.name] = schema
     # Load models from each model test using Runner.prepare_model_data
     # Need to grab associated nodes
-    attrs: Dict[str, Dict[str, List[Any]]] = dict()
-    model_paths: List[Any] = []
+    attrs: dict[str, dict[str, list[Any]]] = {}
+    model_paths: list[Any] = []
     for rt in load_model_tests(kind="real"):
         if rt.url.startswith("onnx/backend/test/data/light/"):
             # testing local files
@@ -192,7 +193,7 @@ def gen_model_test_coverage(
                 # Iterate through and store each node's attributes
                 for attr in node.attribute:
                     if node.op_type not in attrs:
-                        attrs[node.op_type] = dict()
+                        attrs[node.op_type] = {}
                     if attr.name not in attrs[node.op_type]:
                         attrs[node.op_type][attr.name] = []
                     if attr.type == AttributeProto.FLOAT:
@@ -251,7 +252,7 @@ def gen_model_test_coverage(
 
 
 def gen_overall_test_coverage(
-    schemas: Sequence[defs.OpSchema], f: IO[Any], ml: bool
+    schemas: Sequence[defs.OpSchema], f: IO[Any], ml: bool  # noqa: ARG001
 ) -> None:
     f.write("# Overall Test Coverage\n")
     f.write("## To be filled.\n")
