@@ -9,6 +9,7 @@
 
 #pragma once
 #include <stdint.h>
+
 #include <string>
 #include <unordered_map>
 #include <vector>
@@ -184,7 +185,13 @@ namespace ONNX_NAMESPACE {
   _(ScatterElements)                \
   _(Resize)                         \
   _(ceil_mode)                      \
-  _(num_outputs)
+  _(num_outputs)                    \
+  _(start)                          \
+  _(end)                            \
+  _(num_groups)                     \
+  _(stash_type)                     \
+  _(block_size)                     \
+  _(output_dtype)
 
 enum BuiltinSymbol {
 #define DEFINE_SYMBOL(s) k##s,
