@@ -1,17 +1,24 @@
 # Copyright (c) ONNX Project Contributors
 #
 # SPDX-License-Identifier: Apache-2.0
+from __future__ import annotations
 
-# pylint: disable=unidiomatic-typecheck
+from typing import MutableMapping
 
-from typing import Dict, List, MutableMapping, Optional, Set, Tuple
-
-from onnx import GraphProto, ModelProto, TensorProto, checker, helper, utils
+from onnx import (
+    AttributeProto,
+    GraphProto,
+    ModelProto,
+    TensorProto,
+    checker,
+    helper,
+    utils,
+)
 
 
 def check_overlapping_names(
-    g1: GraphProto, g2: GraphProto, io_map: Optional[List[Tuple[str, str]]] = None
-) -> List[Tuple[str, List[str]]]:
+    g1: GraphProto, g2: GraphProto, io_map: list[tuple[str, str]] | None = None
+) -> list[tuple[str, list[str]]]:
     """Checks whether there are name collisions between two graphs
 
     Returns a list of tuples where the first element represents the member containing overlapping names
@@ -26,20 +33,20 @@ def check_overlapping_names(
     if type(g2) is not GraphProto:
         raise ValueError("g2 argument is not an ONNX graph")
 
-    def _overlapping(c1: List[str], c2: List[str]) -> List[str]:
+    def _overlapping(c1: list[str], c2: list[str]) -> list[str]:
         return list(set(c1) & set(c2))
 
-    def _edge_names(graph: GraphProto, exclude: Optional[Set[str]] = None) -> List[str]:
+    def _edge_names(graph: GraphProto, exclude: set[str] | None = None) -> list[str]:
         if exclude is None:
             exclude = set()
         edges = []
         for n in graph.node:
             for i in n.input:
                 if i != "" and i not in exclude:
-                    edges.append(i)
+                    edges.append(i)  # noqa: PERF401
             for o in n.output:
                 if o != "" and o not in exclude:
-                    edges.append(o)
+                    edges.append(o)  # noqa: PERF401
         return edges
 
     result = []
@@ -50,19 +57,19 @@ def check_overlapping_names(
 
     # Edges already cover input/output
     overlap = _overlapping(_edge_names(g1), _edge_names(g2, exclude=io_map_inputs))
-    if len(overlap) > 0:
+    if overlap:
         result.append(("edge", overlap))
 
     overlap = _overlapping(
         [e.name for e in g1.value_info], [e.name for e in g2.value_info]
     )
-    if len(overlap) > 0:
+    if overlap:
         result.append(("value_info", overlap))
 
     overlap = _overlapping(
         [e.name for e in g1.initializer], [e.name for e in g2.initializer]
     )
-    if len(overlap) > 0:
+    if overlap:
         result.append(("initializer", overlap))
 
     overlap = _overlapping(
@@ -72,22 +79,22 @@ def check_overlapping_names(
         [e.indices.name for e in g1.sparse_initializer],
         [e.indices.name for e in g2.sparse_initializer],
     )
-    if len(overlap) > 0:
+    if overlap:
         result.append(("sparse_initializer", overlap))
 
     return result
 
 
-def merge_graphs(  # pylint: disable=too-many-branches,too-many-statements
+def merge_graphs(
     g1: GraphProto,
     g2: GraphProto,
-    io_map: List[Tuple[str, str]],
-    inputs: Optional[List[str]] = None,
-    outputs: Optional[List[str]] = None,
-    prefix1: Optional[str] = None,
-    prefix2: Optional[str] = None,
-    name: Optional[str] = None,
-    doc_string: Optional[str] = None,
+    io_map: list[tuple[str, str]],
+    inputs: list[str] | None = None,
+    outputs: list[str] | None = None,
+    prefix1: str | None = None,
+    prefix2: str | None = None,
+    name: str | None = None,
+    doc_string: str | None = None,
 ) -> GraphProto:
     """Combines two ONNX graphs into a single one.
 
@@ -162,8 +169,8 @@ def merge_graphs(  # pylint: disable=too-many-branches,too-many-statements
             ]
 
         if not outputs:
-            g1_outputs = [o.name for o in g1.input]
-            g2_outputs = [o.name for o in g2.input]
+            g1_outputs = [o.name for o in g1.output]
+            g2_outputs = [o.name for o in g2.output]
         else:
             output_set = set(outputs)
             g1_outputs = [
@@ -207,12 +214,20 @@ def merge_graphs(  # pylint: disable=too-many-branches,too-many-statements
     g.node.extend(g2.node)
     g2_nodes_end = len(g.node)
 
+    # Search inputs of the subgraph recursively
+    def connect_io(sub_graph: GraphProto, start: int, end: int) -> None:
+        for node_idx in range(start, end):
+            node = sub_graph.node[node_idx]
+            for attr in node.attribute:
+                if attr.type == AttributeProto.GRAPH:
+                    connect_io(attr.g, 0, len(attr.g.node))
+
+            for index, name_ in enumerate(node.input):
+                if name_ in reversed_io_map:
+                    node.input[index] = reversed_io_map[name_]
+
     # Connecting outputs of the first graph with the inputs of the second
-    for node_idx in range(g2_nodes_begin, g2_nodes_end):
-        node = g.node[node_idx]
-        for index, name_ in enumerate(node.input):
-            if name_ in reversed_io_map:
-                node.input[index] = reversed_io_map[name_]
+    connect_io(g, g2_nodes_begin, g2_nodes_end)
 
     if inputs:
         input_set = set(inputs)
@@ -265,20 +280,20 @@ def merge_graphs(  # pylint: disable=too-many-branches,too-many-statements
     return g
 
 
-def merge_models(  # pylint: disable=too-many-branches
+def merge_models(
     m1: ModelProto,
     m2: ModelProto,
-    io_map: List[Tuple[str, str]],
-    inputs: Optional[List[str]] = None,
-    outputs: Optional[List[str]] = None,
-    prefix1: Optional[str] = None,
-    prefix2: Optional[str] = None,
-    name: Optional[str] = None,
-    doc_string: Optional[str] = None,
-    producer_name: Optional[str] = "onnx.compose.merge_models",
-    producer_version: Optional[str] = "1.0",
-    domain: Optional[str] = "",
-    model_version: Optional[int] = 1,
+    io_map: list[tuple[str, str]],
+    inputs: list[str] | None = None,
+    outputs: list[str] | None = None,
+    prefix1: str | None = None,
+    prefix2: str | None = None,
+    name: str | None = None,
+    doc_string: str | None = None,
+    producer_name: str | None = "onnx.compose.merge_models",
+    producer_version: str | None = "1.0",
+    domain: str | None = "",
+    model_version: int | None = 1,
 ) -> ModelProto:
     """Combines two ONNX models into a single one.
 
@@ -411,17 +426,17 @@ def merge_models(  # pylint: disable=too-many-branches
     return model
 
 
-def add_prefix_graph(  # pylint: disable=too-many-branches
+def add_prefix_graph(
     graph: GraphProto,
     prefix: str,
-    rename_nodes: Optional[bool] = True,
-    rename_edges: Optional[bool] = True,
-    rename_inputs: Optional[bool] = True,
-    rename_outputs: Optional[bool] = True,
-    rename_initializers: Optional[bool] = True,
-    rename_value_infos: Optional[bool] = True,
-    inplace: Optional[bool] = False,
-    name_map: Optional[Dict[str, str]] = None,
+    rename_nodes: bool | None = True,
+    rename_edges: bool | None = True,
+    rename_inputs: bool | None = True,
+    rename_outputs: bool | None = True,
+    rename_initializers: bool | None = True,
+    rename_value_infos: bool | None = True,
+    inplace: bool | None = False,
+    name_map: dict[str, str] | None = None,
 ) -> GraphProto:
     """Adds a prefix to names of elements in a graph: nodes, edges, inputs, outputs,
     initializers, sparse initializer, value infos.
@@ -531,14 +546,14 @@ def add_prefix_graph(  # pylint: disable=too-many-branches
 def add_prefix(
     model: ModelProto,
     prefix: str,
-    rename_nodes: Optional[bool] = True,
-    rename_edges: Optional[bool] = True,
-    rename_inputs: Optional[bool] = True,
-    rename_outputs: Optional[bool] = True,
-    rename_initializers: Optional[bool] = True,
-    rename_value_infos: Optional[bool] = True,
-    rename_functions: Optional[bool] = True,
-    inplace: Optional[bool] = False,
+    rename_nodes: bool | None = True,
+    rename_edges: bool | None = True,
+    rename_inputs: bool | None = True,
+    rename_outputs: bool | None = True,
+    rename_initializers: bool | None = True,
+    rename_value_infos: bool | None = True,
+    rename_functions: bool | None = True,
+    inplace: bool | None = False,
 ) -> ModelProto:
     """Adds a prefix to names of elements in a graph: nodes, edges, inputs, outputs,
     initializers, sparse initializer, value infos, and local functions.
@@ -605,7 +620,7 @@ def add_prefix(
 def expand_out_dim_graph(
     graph: GraphProto,
     dim_idx: int,
-    inplace: Optional[bool] = False,
+    inplace: bool | None = False,
 ) -> GraphProto:
     """Inserts an extra dimension with extent 1 to each output in the graph.
 
@@ -685,7 +700,7 @@ def expand_out_dim_graph(
 def expand_out_dim(
     model: ModelProto,
     dim_idx: int,
-    inplace: Optional[bool] = False,
+    inplace: bool | None = False,
 ) -> ModelProto:
     """Inserts an extra dimension with extent 1 to each output in the graph.
 
