@@ -1,5 +1,6 @@
-// Copyright (c) ONNX Project Contributors.
-// Licensed under the MIT license.
+/*
+ * SPDX-License-Identifier: Apache-2.0
+ */
 
 #pragma once
 
@@ -12,8 +13,7 @@ namespace ONNX_NAMESPACE {
 // Iterate over schema from ai.onnx.training version 1
 class OpSet_OnnxTraining_ver1 {
  public:
-  static void ForEachSchema(std::function<void(OpSchema&&)> /* fn */) {
-  }
+  static void ForEachSchema(std::function<void(OpSchema&&)> /* fn */) {}
 };
 
 // Register training operators.
@@ -21,4 +21,4 @@ inline void RegisterOnnxTrainingOperatorSetSchema() {
   RegisterOpSetSchema<OpSet_OnnxTraining_ver1>();
 }
 
-} // namespace ONNX_NAMESPACE
\ No newline at end of file
+} // namespace ONNX_NAMESPACE
