@@ -1,23 +1,27 @@
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=C0415,R0912
 
-from abc import ABC, abstractmethod
-from typing import Any, Dict, Iterable, List, Optional, Tuple
+
+from __future__ import annotations
+
+import abc
+from typing import Any, ClassVar, Iterable
 
 import numpy as np
 
 from onnx import TensorProto
-from onnx.defs import get_all_schemas_with_history, get_schema, onnx_opset_version
-from onnx.helper import make_node
-from onnx.numpy_helper import to_array
-from onnx.onnx_pb import AttributeProto, GraphProto, NodeProto, TypeProto
-from onnx.reference.custom_element_types import (
+from onnx._custom_element_types import (
     bfloat16,
     float8e4m3fn,
     float8e4m3fnuz,
     float8e5m2,
     float8e5m2fnuz,
+    int4,
+    uint4,
 )
+from onnx.defs import get_all_schemas_with_history, get_schema, onnx_opset_version
+from onnx.helper import make_node, make_tensor_type_proto, np_dtype_to_tensor_dtype
+from onnx.numpy_helper import to_array
+from onnx.onnx_pb import AttributeProto, GraphProto, NodeProto, TypeProto
 
 
 def _split_class_name(name):  # type: ignore
@@ -32,48 +36,37 @@ def _split_class_name(name):  # type: ignore
 
 
 class RuntimeTypeError(RuntimeError):
-    """
-    Raised when a type of a variable is unexpected.
-    """
+    """Raised when a type of a variable is unexpected."""
 
 
 class RuntimeContextError(RuntimeError):
-    """
-    Raised when the context is missing but an context dependent implementation is defined for an operator.
-    """
+    """Raised when the context is missing but an context dependent implementation is defined for an operator."""
 
 
 class RuntimeImplementationError(NotImplementedError):
-    """
-    Raised when no implementation was found for an operator.
-    """
+    """Raised when no implementation was found for an operator."""
 
 
 class DefaultNone:
-    """
-    Default value for parameters when the parameter is not set
-    but the operator has a default behavior for it.
-    """
+    """Default value for parameters when the parameter is not set but the operator has a default behavior for it."""
 
 
 class RefAttrName:
-    """
-    Implements a link between a parameter of a function
-    and an attribute in node.
+    """Implements a link between a parameter of a function and an attribute in node.
 
-    :param name: name of the input
+    Args:
+        name: name of the input
     """
 
     def __init__(self, name: str):
         self.name = name
 
     def __repr__(self) -> str:
-        "usual"
         return f"{self.__class__.__name__}({self.name!r})"
 
 
-def _build_schemas() -> Dict[str, type]:
-    res: Dict[str, type] = {}
+def _build_schemas() -> dict[str, type]:
+    res: dict[str, type] = {}
     for schema in get_all_schemas_with_history():
         # Multiple version can coexist. The last one is kept.
         if schema.name in res:
@@ -105,13 +98,12 @@ class OnnxType:
 
 
 class SparseTensor:
-    """
-    Simple representation of a sparse tensor.
+    """Simple representation of a sparse tensor.
     It is based on numpy but does not require scipy.
     """
 
     def __init__(
-        self, values: np.ndarray, indices: np.ndarray, shape: Tuple[int]
+        self, values: np.ndarray, indices: np.ndarray, shape: tuple[int]
     ) -> None:
         self.values = values
         self.indices = indices
@@ -123,70 +115,37 @@ class SparseTensor:
 
 
 def to_sparse_tensor(att: AttributeProto) -> SparseTensor:
-    """
-    Hosts a sparse tensor.
-    """
+    """Hosts a sparse tensor."""
     shape = tuple(d for d in att.dims)  # type: ignore[attr-defined]
     return SparseTensor(to_array(att.values), to_array(att.indices), shape)  # type: ignore
 
 
 def to_array_extended(tensor: TensorProto) -> np.ndarray:
-    """
-    Similar to :func:`to_array` but deals with bfloat16,
-    float8e4m3fn, float8e4m3fnuz, float8e5m2, float8e5m2fnuz.
-    """
-    elem_type = tensor.data_type
-    if elem_type == TensorProto.BFLOAT16:
-        data = tensor.int32_data
-        shape = tuple(tensor.dims)
-        y = np.empty(shape, dtype=bfloat16).ravel()
-        for i, d in enumerate(data):
-            y[i] = d
-        return y.reshape(shape)
-
-    if elem_type in (
-        TensorProto.FLOAT8E4M3FN,
-        TensorProto.FLOAT8E4M3FNUZ,
-        TensorProto.FLOAT8E5M2,
-        TensorProto.FLOAT8E5M2FNUZ,
-    ):
-        m = {
-            TensorProto.FLOAT8E4M3FN: float8e4m3fn,
-            TensorProto.FLOAT8E4M3FNUZ: float8e4m3fnuz,
-            TensorProto.FLOAT8E5M2: float8e5m2,
-            TensorProto.FLOAT8E5M2FNUZ: float8e5m2fnuz,
-        }
-
-        data = tensor.int32_data
-        shape = tuple(tensor.dims)
-        y = np.empty(shape, dtype=m[elem_type]).ravel()  # type: ignore[index]
-        for i, d in enumerate(data):
-            y[i] = d
-        return y.reshape(shape)
+    """Alias for :func:`to_array`."""
     return to_array(tensor)
 
 
 class Graph:
-    __slots__ = ["g"]
+    __slots__ = ("g",)
 
     def __init__(self, g: GraphProto) -> None:
         self.g = g
 
 
-class OpRun(ABC):
-    """
-    Ancestor to all operators in this subfolder.
+class OpRun(abc.ABC):
+    """Ancestor to all operators in this subfolder.
 
-    :param onnx_node: `onnx` node
-    :param run_params: additional parameters such as `verbose`, `opsets`
-        (it can be more than one if the operator has a subgraph),
-        `log` for a logging function
-    :param schema: operator schema
+    Args:
+        onnx_node: `onnx` node
+        run_params: additional parameters such as `verbose`, `opsets`
+            (it can be more than one if the operator has a subgraph),
+            `log` for a logging function
+        schema: operator schema
     """
 
     op_domain = ""
 
-    _attribute_conversion_functions = {
+    _attribute_conversion_functions: ClassVar[dict[Any, Any]] = {
         AttributeProto.FLOAT: lambda att: np.float32(att.f),
         AttributeProto.FLOATS: lambda att: [np.float32(f) for f in att.floats],
         AttributeProto.GRAPH: lambda att: Graph(att.g),
@@ -206,7 +165,7 @@ class OpRun(ABC):
     }
 
     def __init__(
-        self, onnx_node: NodeProto, run_params: Dict[str, Any], schema: Any = None
+        self, onnx_node: NodeProto, run_params: dict[str, Any], schema: Any = None
     ):
         if not isinstance(run_params, dict):
             raise TypeError(f"run_params must be a dictionary not {type(run_params)}.")
@@ -214,7 +173,7 @@ class OpRun(ABC):
             if att not in run_params:
                 raise RuntimeError(
                     f"Attribute {att!r} must be in run_params, only "
-                    f"{list(sorted(run_params))} was found."
+                    f"{sorted(run_params)} was found."
                 )
         if "log" not in run_params:
             raise KeyError("run_params must contains key 'log'.")
@@ -238,22 +197,25 @@ class OpRun(ABC):
         self.run_params["log"](pattern, *args)
 
     def _extract_attribute_value(
-        self, att: AttributeProto, ref_att: Optional[AttributeProto] = None
+        self, att: AttributeProto, ref_att: AttributeProto | None = None
     ) -> Any:
-        """
-        Converts an attribute value into a python value.
-        """
+        """Converts an attribute value into a python value."""
         if att.type == AttributeProto.GRAPH:
-            from onnx.reference.reference_evaluator import (
-                ReferenceEvaluator,  # type: ignore
-            )
-
             new_ops = self.run_params.get("new_ops", None)
-            return ReferenceEvaluator(
+            if "existing_functions" in self.run_params:
+                functions = list(self.run_params["existing_functions"].values())
+            else:
+                functions = None
+            evaluator_cls = self.run_params.get("evaluator_cls", None)
+            assert (
+                evaluator_cls is not None
+            ), f"evaluator_cls must be specified to evaluate att={att}"
+            return evaluator_cls(
                 att.g,
                 opsets=self.run_params["opsets"],
                 verbose=max(0, self.run_params.get("verbose", 0) - 2),
-                new_ops=None if new_ops is None else new_ops.values(),
+                new_ops=None if new_ops is None else list(new_ops.values()),
+                functions=functions,
             )
         if att.type in OpRun._attribute_conversion_functions:
             return OpRun._attribute_conversion_functions[att.type](att)  # type: ignore
@@ -274,7 +236,7 @@ class OpRun(ABC):
         return value.run(None, context or {}, attributes=attributes)
 
     def _load_attributes(self) -> None:
-        "Checks and loads attributes."
+        """Checks and loads attributes."""
         self.has_linked_attribute = False
         added_attributes = []
         for att in self.onnx_node.attribute:
@@ -306,7 +268,10 @@ class OpRun(ABC):
                             f"for node {self.onnx_node.op_type!r}."
                         )
                     if hasattr(v, "default_value"):
-                        if v.default_value.type == 0:
+                        if v.default_value.type == 0 or (
+                            v.default_value.type == 4  # noqa: PLR2004
+                            and v.default_value.t.data_type == 0
+                        ):
                             # default value is undefined, it depends on the inputs
                             value = None  # type: ignore
                         else:
@@ -316,9 +281,8 @@ class OpRun(ABC):
         self.attributes_names_ = set(added_attributes)
 
     @staticmethod
-    def implicit_inputs(graph: GraphProto) -> List[str]:
-        """
-        Returns all varibles not registered as inputs and not produced by
+    def implicit_inputs(graph: GraphProto) -> list[str]:
+        """Returns all varibles not registered as inputs and not produced by
         an node inside the graph. This inputs are part of the context
         existing in the graph calling this one.
         """
@@ -342,27 +306,26 @@ class OpRun(ABC):
 
     @property
     def input(self) -> Iterable[str]:
-        "Returns node attribute `input`."
+        """Returns node attribute `input`."""
         return self.onnx_node.input  # type: ignore
 
     @property
     def output(self) -> Iterable[str]:
-        "Returns node attribute `output`."
+        """Returns node attribute `output`."""
         return self.onnx_node.output  # type: ignore
 
     @property
     def op_type(self) -> str:
-        "Returns node attribute `op_type`."
+        """Returns node attribute `op_type`."""
         return self.onnx_node.op_type  # type: ignore
 
     @property
     def domain(self) -> str:
-        "Returns node attribute `domain`."
+        """Returns node attribute `domain`."""
         return self.onnx_node.domain  # type: ignore
 
     def need_context(self) -> bool:
-        """
-        Tells the runtime if this node needs the context
+        """Tells the runtime if this node needs the context
         (all the results produced so far) as it may silently access
         one of them (operator Scan, If, Loop).
         The default answer is `False`.
@@ -379,33 +342,69 @@ class OpRun(ABC):
         atts.append(")")
         return "\n".join(atts)
 
-    @abstractmethod
+    @abc.abstractmethod
     def _run(self, *args, **kwargs):  # type: ignore
-        """
-        Should be overwritten.
+        """Should be overwritten.
 
-        :param args: operator inputs
-        :param kwargs: optional inputs and overriden attributes,
-            an attribute may be overridden if it belongs to a function,
-            in this case, the same instance of OpRun can be called
-            with different values of the same attribute.
-        :return: outputs
+        Args:
+            *args: operator inputs
+            **kwargs: optional inputs and overriden attributes, an
+                attribute may be overridden if it belongs to a function,
+                in this case, the same instance of OpRun can be called
+                with different values of the same attribute.
+
+        Returns:
+            outputs
         """
         raise NotImplementedError(
             f"Method '_run' must be overwritten for operator {self.__class__.__name__!r}."
         )
 
+    def _check_and_fix_outputs(self, res: tuple[Any, ...]) -> tuple[Any, ...]:
+        """Checks the output are from the expected type."""
+        if not isinstance(res, tuple):
+            raise TypeError(
+                f"Method '_run' of class {self.__class__.__name__!r} does not return a tuple but '{type(res)}'."
+            )
+        if not res:
+            raise ValueError(
+                f"Method '_run' of class {self.__class__.__name__!r} does not return any result."
+            )
+        if any(isinstance(t, tuple) for t in res):
+            dtypes = [type(t) for t in res]
+            raise TypeError(
+                f"One of the results returned by method '_run' of class {self.__class__.__name__!r} "
+                f"is a tuple, this is no ONNX corresponding type (Map, List, Tensor, SparseTensor). "
+                f"All returned types: {dtypes!r}."
+            )
+        res = tuple(  # type: ignore[assignment]
+            (np.array(x) if np.isscalar(x) else x) for x in res
+        )
+        if any(
+            not (isinstance(t, (np.ndarray, list, dict)) or hasattr(t, "todense"))
+            for t in res
+        ):
+            dtypes = [type(t) for t in res]
+            raise TypeError(
+                f"One of the results returned by method '_run' of class {self.__class__.__name__!r} "
+                f"has an unexpected type, this is no ONNX correponding type (Map, List, Tensor, SparseTensor). "
+                f"All returned types: {dtypes!r}."
+            )
+        return res
+
     def run(self, *args, linked_attributes=None, context=None):  # type: ignore
-        """
-        Calls method ``_run``, catches exceptions,
+        """Calls method ``_run``, catches exceptions,
         displays a longer error message.
 
-        :param args: inputs
-        :param linked_attributes: used if this has an attriute linked
-            to the attribute of the function it belongs to
-        :param context: if this node is part of the subgraph, `context`
-            is a dictionary with the values this node may use
-        :return: tuple of results
+        Args:
+            *args: inputs
+            linked_attributes: used if this has an attriute linked to
+                the attribute of the function it belongs to
+            context: if this node is part of the subgraph, `context` is
+                a dictionary with the values this node may use
+
+        Returns:
+            tuple of results
         """
         if self.need_context():
             if context is None:
@@ -453,7 +452,7 @@ class OpRun(ABC):
                 )
             kwargs[att] = getattr(self, att)
         if self.has_subgraph:
-            if self.has_linked_attribute and len(linked_attributes) == 0:
+            if self.has_linked_attribute and not linked_attributes:
                 raise RuntimeError(
                     f"A subgraph has linked attribute but none was given to {type(self)}."
                 )
@@ -461,49 +460,54 @@ class OpRun(ABC):
         if context is not None:
             kwargs["context"] = context
         try:
-            if len(overridden_attributes) > 0:
+            if overridden_attributes:
                 res = self._run(*args, **overridden_attributes, **kwargs)
             else:
                 res = self._run(*args, **kwargs)
         except (TypeError, AttributeError) as e:
             raise TypeError(
                 f"Issues with types {[type(_) for _ in args]} and attributes "
-                f"{list(sorted(kwargs))} and linked attributes={list(sorted(overridden_attributes))} "
+                f"{sorted(kwargs)} and linked attributes={sorted(overridden_attributes)} "
                 f"(operator {self.__class__.__name__!r})."
             ) from e
-        self._log("-- done %s.run -> %d outputs", self.__class__.__name__, len(res))
-        if not isinstance(res, tuple):
-            raise TypeError(
-                f"Method '_run' of class {self.__class__.__name__!r} does not return a tuple but {type(res)}."
-            )
-        if len(res) == 0:
-            raise ValueError(
-                f"Method '_run' of class {self.__class__.__name__!r} does not return any result."
-            )
-        if any(map(lambda t: isinstance(t, tuple), res)):
-            dtypes = [type(t) for t in res]
-            raise TypeError(
-                f"One of the results returned by method '_run' of class {self.__class__.__name__!r} "
-                f"is a tuple, this is no onnx correponding type (Map, List, Tensor, SparseTensor). "
-                f"All returned types: {dtypes!r}."
-            )
-        return res
+        self._log(
+            "-- done %s.run -> %d outputs",
+            self.__class__.__name__,
+            len(res) if res is not None else 0,
+        )
+        return self._check_and_fix_outputs(res)
+
+    @classmethod
+    def infer_name(cls):
+        name = cls.__name__
+        if "_" not in name:
+            return name, onnx_opset_version()
+        name, vers = name.rsplit("_", 1)
+        try:
+            i_vers = int(vers)
+        except ValueError:
+            return cls.__name__, onnx_opset_version()
+        return name, i_vers
 
     @classmethod
     def make_node(
         cls,
-        n_inputs: Optional[int] = None,
-        n_outputs: Optional[int] = None,
+        n_inputs: int | None = None,
+        n_outputs: int | None = None,
         **kwargs: Any,
     ) -> NodeProto:  # type: ignore
-        """
-        Creates an ONNX node for this class based on the given information.
+        """Creates an ONNX node for this class based on the given information.
 
-        :param n_inputs: number of inputs (default is defined by the operator schema)
-        :param n_outputs: number of outputs (default is defined by the operator schema)
-        :param verbose: verbosity
-        :param kwargs: node attributes
-        :return: NodeProto
+        Args:
+            n_inputs: number of inputs (default is defined by the
+                operator schema)
+            n_outputs: number of outputs (default is defined by the
+                operator schema)
+            verbose: verbosity
+            **kwargs: node attributes
+
+        Returns:
+            NodeProto
 
         Method :meth:`eval <onnx.reference.op_run.OpRun.eval>` creates an onnx node
         returned by method :meth:`make_node <onnx.reference.op_run.OpRun.make_node>`.
@@ -516,38 +520,43 @@ class OpRun(ABC):
             onnx_node = Celu.make_node(alpha=0.5)
             print(onnx_node)
         """
+        op_type, opset = cls.infer_name()
         domain = cls.op_domain
         schema = None
         if n_inputs is None:
             if schema is None:
-                schema = get_schema(cls.__name__, onnx_opset_version(), domain)
+                schema = get_schema(op_type, opset, domain)
             n_inputs = schema.min_input
         if n_outputs is None:
             if schema is None:
-                schema = get_schema(cls.__name__, onnx_opset_version(), domain)
+                schema = get_schema(op_type, opset, domain)
             n_outputs = schema.min_output
 
         names_in = [f"x{i}" for i in range(n_inputs)]
         names_out = [f"y{i}" for i in range(n_outputs)]
-        node = make_node(cls.__name__, names_in, names_out, **kwargs)
+        node = make_node(op_type, names_in, names_out, **kwargs)
         return node
 
     @classmethod
     def create(
         cls,
-        n_inputs: Optional[int] = None,
-        n_outputs: Optional[int] = None,
+        n_inputs: int | None = None,
+        n_outputs: int | None = None,
         verbose: int = 0,
         **kwargs: Any,
     ) -> Any:
-        """
-        Instantiates this class based on the given information.
+        """Instantiates this class based on the given information.
 
-        :param n_inputs: number of inputs (default is defined by the operator schema)
-        :param n_outputs: number of outputs (default is defined by the operator schema)
-        :param verbose: verbosity
-        :param kwargs: node attributes
-        :return: NodeProto
+        Args:
+            n_inputs: number of inputs (default is defined by the
+                operator schema)
+            n_outputs: number of outputs (default is defined by the
+                operator schema)
+            verbose: verbosity
+            **kwargs: node attributes
+
+        Returns:
+            NodeProto
         """
 
         def log_function(pattern: str, *args: Any) -> None:
@@ -555,31 +564,34 @@ class OpRun(ABC):
                 print(pattern % tuple(args))
 
         node = cls.make_node(n_inputs, n_outputs, **kwargs)
-        run_params = dict(
-            verbose=verbose,
-            log=log_function,
-            new_ops=None,
-            opsets={"": onnx_opset_version()},
-        )
+        run_params = {
+            "verbose": verbose,
+            "log": log_function,
+            "new_ops": None,
+            "opsets": {"": onnx_opset_version()},
+        }
         cl = cls(node, run_params)
         return cl
 
     @classmethod
     def eval(
         cls,
-        *args: List[Any],
-        n_outputs: Optional[int] = None,
+        *args: list[Any],
+        n_outputs: int | None = None,
         verbose: int = 0,
         **kwargs: Any,
     ) -> Any:  # type: ignore
-        """
-        Evaluates this operator.
+        """Evaluates this operator.
 
-        :param args: inputs
-        :param n_outputs: number of outputs (default is defined by the operator schema)
-        :param verbose: verbosity
-        :param kwargs: node attributes
-        :return: NodeProto
+        Args:
+            *args: inputs
+            n_outputs: number of outputs (default is defined by the
+                operator schema)
+            verbose: verbosity
+            **kwargs: node attributes
+
+        Returns:
+            NodeProto
         """
         inst = cls.create(len(args), n_outputs=n_outputs, verbose=verbose, **kwargs)
         res = inst.run(*args)
@@ -588,40 +600,114 @@ class OpRun(ABC):
         return res
 
 
-class OpFunction(OpRun):
-    """
-    Runs a custom function.
-    """
+class OpRunExpand(OpRun):
+    """Class any operator to avoid must inherit from."""
 
-    def __init__(self, onnx_node: NodeProto, log_function: Any, impl: Any = None):
+    def __init__(
+        self,
+        onnx_node: NodeProto,  # noqa: ARG002
+        run_params: dict[str, Any],  # noqa: ARG002
+        impl: Any = None,  # noqa: ARG002
+    ):
+        raise RuntimeError(
+            f"The reference implementation must not use this node ({type(self)})."
+        )
+
+    def _run(self, *inputs, **kwargs):  # noqa: ARG002
+        raise RuntimeError(
+            f"The reference implementation must not use this node ({type(self)})."
+        )
+
+
+class OpFunction(OpRun):
+    """Runs a custom function."""
+
+    def __init__(
+        self,
+        onnx_node: NodeProto,
+        run_params: dict[str, Any] | None,
+        impl: Any = None,
+        attributes: dict[str, Any] | None = None,
+    ):
         if impl is None:
             raise RuntimeError(
                 f"impl cannot be None for node type {onnx_node.op_type!r} "
                 f"from domain {onnx_node.domain!r}."
             )
-        OpRun.__init__(self, onnx_node, log_function)
+        OpRun.__init__(self, onnx_node, run_params)  # type: ignore[arg-type]
         self.impl_ = impl
         # The function implementation is the same whenever the function is called
         # but the attributes may be different at every call.
         self.attributes_ = {
-            name: getattr(self, name) for name in self.impl_.attributes_
+            name: getattr(self, name)
+            for name in getattr(self.impl_, "attributes_", attributes)  # type: ignore[union-attr]
         }
 
-    def _run(self, *inputs, **kwargs):  # type: ignore # pylint: disable=W0221
-        if len(self.impl_.input_names) != len(inputs):
+    def _run(self, *inputs, **kwargs):  # type: ignore
+        return self._run_impl(self.impl_, *inputs, **kwargs)
+
+    def _run_impl(self, impl, *inputs, **kwargs):  # type: ignore
+        if len(impl.input_names) != len(inputs):
             raise RuntimeError(
                 f"Mismatch lengths between the number of inputs {len(inputs)} "
-                f"and the expected number of inputs {len(self.impl_.inputs)} "
+                f"and the expected number of inputs {len(impl.inputs)} "
                 f"for node {self.op_type!r} from domain {self.domain!r}."
             )
-        feeds = dict(zip(self.impl_.input_names, inputs))
+        feeds = dict(zip(impl.input_names, inputs))
         attributes = self.attributes_.copy()
         attributes.update(kwargs)
-        results = self.impl_.run(None, feeds, attributes=attributes)
-        if len(self.impl_.output_names) != len(results):
+        results = impl.run(None, feeds, attributes=attributes)
+        if len(impl.output_names) != len(results):
             raise RuntimeError(
                 f"Mismatch lengths between the number of outputs {len(results)} "
-                f"and the expected number of outputs {len(self.impl_.output_names)} "
+                f"and the expected number of outputs {len(impl.output_names)} "
                 f"for node {self.op_type!r} from domain {self.domain!r}."
             )
         return tuple(results)
+
+
+class OpFunctionContextDependant(OpFunction):
+    """The function can be instantiated but only at execution time.
+    An instance of OpFunction is created everytime to node is executed.
+    This is needed when the schema of an operator defines a context dependant function.
+    """
+
+    def __init__(
+        self,
+        onnx_node: NodeProto,
+        run_params: dict[str, Any] | None,
+        parent: Any = None,
+    ):
+        OpFunction.__init__(self, onnx_node, run_params, impl=self, attributes={})
+        self.parent = parent
+        version = parent.opsets[onnx_node.domain]
+        self.schema_ = get_schema(onnx_node.op_type, version, onnx_node.domain)
+
+    def _run(self, *inputs, **kwargs):
+        # Input types are known. They are used to properly
+        # created the body for this operator.
+        types = []
+        for t in inputs:
+            try:
+                ttype = np_dtype_to_tensor_dtype(t.dtype)
+            except KeyError:
+                if t.dtype == float8e4m3fn:
+                    ttype = TensorProto.FLOAT8E4M3FN  # type: ignore[attr-defined]
+                elif t.dtype == float8e4m3fnuz:
+                    ttype = TensorProto.FLOAT8E4M3FNUZ  # type: ignore[attr-defined]
+                elif t.dtype == float8e5m2:
+                    ttype = TensorProto.FLOAT8E5M2  # type: ignore[attr-defined]
+                elif t.dtype == float8e5m2fnuz:
+                    ttype = TensorProto.FLOAT8E5M2FNUZ  # type: ignore[attr-defined]
+                elif t.dtype == bfloat16:
+                    ttype = TensorProto.BLOFAT16  # type: ignore[attr-defined]
+                elif t.dtype == uint4:
+                    ttype = TensorProto.UINT4  # type: ignore[attr-defined]
+                elif t.dtype == int4:
+                    ttype = TensorProto.INT4  # type: ignore[attr-defined]
+                else:
+                    raise
+            types.append(make_tensor_type_proto(ttype, t.shape))
+        cl = self.parent._load_impl(self.onnx_node, types)
+        inst = cl(self.onnx_node, self.run_params)
+        return self._run_impl(inst.impl_, *inputs, **kwargs)
