@@ -1,16 +1,14 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=R0912,R0913,R0914,W0221
+from __future__ import annotations
 
 from onnx.reference.ops.aionnxml._op_run_aionnxml import OpRunAiOnnxMl
 from onnx.reference.ops.aionnxml.op_svm_helper import SVMCommon
 
 
 class SVMRegressor(OpRunAiOnnxMl):
-    """
-    The class only implements `POST_TRANSFORM="NONE"`.
-    """
+    """The class only implements `POST_TRANSFORM="NONE"`."""
 
     def _run(  # type: ignore
         self,
@@ -37,7 +35,7 @@ class SVMRegressor(OpRunAiOnnxMl):
             support_vectors=support_vectors,
         )
         # adding an attribute for debugging purpose
-        self._svm = svm  # pylint: disable=W0201
+        self._svm = svm
         res = svm.run_reg(X)
 
         if post_transform in (None, "NONE"):
