@@ -1,10 +1,10 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=C0415,R0912,W0611,W0603
+from __future__ import annotations
 
 import textwrap
-from typing import Any, Dict, Union
+from typing import Any
 
 from onnx.reference.op_run import OpFunction
 from onnx.reference.ops._helpers import build_registered_operators_any_domain
@@ -12,25 +12,30 @@ from onnx.reference.ops.experimental._op_run_experimental import OpRunExperiment
 from onnx.reference.ops.experimental.op_im2col import Im2Col  # noqa: F401
 
 
-def _build_registered_operators() -> (
-    Dict[str, Dict[Union[int, None], OpRunExperimental]]
-):
+def _build_registered_operators() -> dict[str, dict[int | None, OpRunExperimental]]:
     return build_registered_operators_any_domain(globals().copy())  # type: ignore[return-value]
 
 
 def load_op(
-    domain: str, op_type: str, version: Union[None, int], custom: Any = None
+    domain: str,
+    op_type: str,
+    version: None | int,
+    custom: Any = None,
+    evaluator_cls: type | None = None,  # noqa: ARG001
 ) -> Any:
-    """
-    Loads the implemented for a specified operator.
+    """Loads the implemented for a specified operator.
 
-    :param domain: domain
-    :param op_type: oprator type
-    :param version: requested version
-    :param custom: custom implementation (like a function)
-    :return: class
+    Args:
+        domain: domain
+        op_type: oprator type
+        version: requested version
+        custom: custom implementation (like a function)
+        evaluator_cls: unused
+
+    Returns:
+        class
     """
-    global _registered_operators
+    global _registered_operators  # noqa: PLW0603
     if _registered_operators is None:
         _registered_operators = _build_registered_operators()  # type: ignore[assignment]
     if custom is not None:
@@ -75,6 +80,4 @@ def load_op(
     return cl
 
 
-# Python 3.7 does not support this annotation for a global variable.
-# _registered_operators: TOptional[Dict[str, Dict[Union[int, None], OpRunExperimental]]] = None
-_registered_operators = None  # type: ignore
+_registered_operators: dict[str, dict[int | None, OpRunExperimental]] | None = None
