@@ -1,5 +1,7 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=W0221
+from __future__ import annotations
 
 import numpy as np
 
@@ -7,15 +9,14 @@ from onnx.reference.op_run import OpRun
 
 
 class Concat(OpRun):
-    def _preprocess(self, a: np.ndarray) -> np.ndarray:
+    def _preprocess(self, a: np.ndarray, axis: int) -> np.ndarray:
         if len(a.shape) == 0:
             raise RuntimeError(f"Concat: one input has an empty shape: {a!r}.")
-        if self.axis >= len(a.shape):  # type: ignore
-            new_shape = a.shape + (1,) * (self.axis + 1 - len(a.shape))  # type: ignore
+        if axis >= len(a.shape):  # type: ignore
+            new_shape = a.shape + (1,) * (axis + 1 - len(a.shape))  # type: ignore
             return a.reshape(new_shape)
         return a
 
     def _run(self, *args, axis=None):  # type: ignore
-        axis = axis or self.axis  # type: ignore
-        targs = tuple(self._preprocess(a) for a in args)
+        targs = tuple(self._preprocess(a, axis) for a in args)
         return (np.concatenate(targs, axis),)  # type: ignore
