@@ -1,16 +1,16 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=W0221
+from __future__ import annotations
 
-from typing import Any, List
+from typing import Any
 
 import numpy as np
 
 from onnx.reference.op_run import OpRun
 
 
-def _concat_from_sequence(seq: List[Any], axis: int, new_axis: int = 0) -> np.ndarray:
+def _concat_from_sequence(seq: list[Any], axis: int, new_axis: int = 0) -> np.ndarray:
     if new_axis == 1:
         seq2 = [s[..., np.newaxis] for s in seq]
         res = np.concatenate(seq2, axis=-1)
