@@ -1,11 +1,11 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=W0221,R0913,R0914
+from __future__ import annotations
 
 import numpy as np
 
-from onnx.reference.ops._op_common_pool import CommonPool
+from onnx.reference.ops.op_pool_common import CommonPool
 
 
 class LpPool(CommonPool):
@@ -38,4 +38,6 @@ class LpPool(CommonPool):
         )
 
         kernel_element_count = np.prod(kernel_shape)
-        return (np.power(kernel_element_count * power_average[0], 1.0 / p),)
+        return (
+            np.power(kernel_element_count * power_average[0], 1.0 / p).astype(x.dtype),
+        )
