@@ -1,10 +1,9 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=R0902,R0911,R0912,R0913,R0914,W0221
+from __future__ import annotations
 
 import dataclasses
-from typing import Optional, Tuple
 
 import numpy as np
 
@@ -13,20 +12,20 @@ from onnx.reference.op_run import OpRun
 
 @dataclasses.dataclass
 class PrepareContext:
-    boxes_data_: Optional[np.ndarray] = None
+    boxes_data_: np.ndarray | None = None
     boxes_size_: int = 0
-    scores_data_: Optional[np.ndarray] = None
+    scores_data_: np.ndarray | None = None
     scores_size_: int = 0
-    max_output_boxes_per_class_: Optional[np.ndarray] = None
-    score_threshold_: Optional[np.ndarray] = None
-    iou_threshold_: Optional[np.ndarray] = None
+    max_output_boxes_per_class_: np.ndarray | None = None
+    score_threshold_: np.ndarray | None = None
+    iou_threshold_: np.ndarray | None = None
     num_batches_: int = 0
     num_classes_: int = 0
     num_boxes_: int = 0
 
 
 class SelectedIndex:
-    __slots__ = ["batch_index_", "class_index_", "box_index_"]
+    __slots__ = ("batch_index_", "class_index_", "box_index_")
 
     def __init__(
         self, batch_index: int = 0, class_index: int = 0, box_index: int = 0
@@ -36,13 +35,13 @@ class SelectedIndex:
         self.box_index_ = box_index
 
 
-def max_min(lhs: float, rhs: float) -> Tuple[float, float]:
+def max_min(lhs: float, rhs: float) -> tuple[float, float]:
     if lhs >= rhs:
         return rhs, lhs
     return lhs, rhs
 
 
-def suppress_by_iou(
+def suppress_by_iou(  # noqa: PLR0911
     boxes_data: np.ndarray,
     box_index1: int,
     box_index2: int,
@@ -133,7 +132,7 @@ class NonMaxSuppression(OpRun):
         max_output_boxes_per_class: int,
         iou_threshold: float,
         score_threshold: float,
-    ) -> Tuple[int, float, float]:
+    ) -> tuple[int, float, float]:
         if pc.max_output_boxes_per_class_ is not None:
             max_output_boxes_per_class = max(pc.max_output_boxes_per_class_[0], 0)
 
@@ -225,16 +224,16 @@ class NonMaxSuppression(OpRun):
                 if pc.score_threshold_ is not None:
                     for box_index in range(pc.num_boxes_):
                         if class_scores[box_index] > score_threshold:
-                            candidate_boxes.append(
+                            candidate_boxes.append(  # noqa: PERF401
                                 BoxInfo(class_scores[box_index], box_index)
                             )
                 else:
                     for box_index in range(pc.num_boxes_):
-                        candidate_boxes.append(
+                        candidate_boxes.append(  # noqa: PERF401
                             BoxInfo(class_scores[box_index], box_index)
                         )
 
-                sorted_boxes = list(sorted(candidate_boxes))
+                sorted_boxes = sorted(candidate_boxes)
 
                 selected_boxes_inside_class = []  # type: ignore
                 # Get the next box with top score, filter by iou_threshold.
