@@ -1,7 +1,7 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=R0913,W0221
+from __future__ import annotations
 
 from onnx.reference.ops._op_common_random import _CommonRandom
 
