@@ -1,9 +1,10 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=W0221
+from __future__ import annotations
 
 import numpy as np
 
-from onnx.defs import onnx_opset_version
 from onnx.reference.op_run import OpRun
 
 
@@ -36,9 +37,3 @@ class Reshape_14(CommonReshape):
         else:
             allowzero = allowzero == 1
         return (reshape_reference_implementation(data, shape, allowzero),)
-
-
-if onnx_opset_version() >= 14:
-    Reshape = Reshape_14
-else:
-    Reshape = Reshape_5  # type: ignore
