@@ -1,9 +1,7 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=R0902,R0912,R0913,R0914,R0915,R1702,W0221
-
-from typing import Tuple
+from __future__ import annotations
 
 import numpy as np
 
@@ -117,7 +115,7 @@ class RoiAlign(OpRun):
 
     @staticmethod
     def roi_align_forward(  # type: ignore
-        output_shape: Tuple[int, int, int, int],
+        output_shape: tuple[int, int, int, int],
         bottom_data,
         spatial_scale,
         height: int,
@@ -235,20 +233,12 @@ class RoiAlign(OpRun):
                                 for _ix in range(roi_bin_grid_w):
                                     pc = pre_calc[pre_calc_index]
                                     val = max(
-                                        max(
-                                            max(
-                                                pc.w1
-                                                * bottom_data[
-                                                    offset_bottom_data + pc.pos1
-                                                ],
-                                                pc.w2
-                                                * bottom_data[
-                                                    offset_bottom_data + pc.pos2
-                                                ],
-                                            ),
-                                            pc.w3
-                                            * bottom_data[offset_bottom_data + pc.pos3],
-                                        ),
+                                        pc.w1
+                                        * bottom_data[offset_bottom_data + pc.pos1],
+                                        pc.w2
+                                        * bottom_data[offset_bottom_data + pc.pos2],
+                                        pc.w3
+                                        * bottom_data[offset_bottom_data + pc.pos3],
                                         pc.w4
                                         * bottom_data[offset_bottom_data + pc.pos4],
                                     )
@@ -273,7 +263,9 @@ class RoiAlign(OpRun):
         sampling_ratio=None,
         spatial_scale=None,
     ):
-        coordinate_transformation_mode = coordinate_transformation_mode or self.coordinate_transformation_mode  # type: ignore
+        coordinate_transformation_mode = (
+            coordinate_transformation_mode or self.coordinate_transformation_mode
+        )  # type: ignore
         mode = mode or self.mode  # type: ignore
         output_height = output_height or self.output_height  # type: ignore
         output_width = output_width or self.output_width  # type: ignore
@@ -301,4 +293,4 @@ class RoiAlign(OpRun):
             coordinate_transformation_mode.lower() == "half_pixel",
             batch_indices.flatten(),
         )
-        return (Y.reshape((y_dims)).astype(X.dtype),)
+        return (Y.reshape(y_dims).astype(X.dtype),)
