@@ -1,9 +1,7 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=R0912,R0913,W0221
-
-from typing import Optional
+from __future__ import annotations
 
 import numpy as np
 
@@ -14,8 +12,8 @@ def _slice(
     data: np.ndarray,
     starts: np.ndarray,
     ends: np.ndarray,
-    axes: Optional[np.ndarray] = None,
-    steps: Optional[np.ndarray] = None,
+    axes: np.ndarray | None = None,
+    steps: np.ndarray | None = None,
 ) -> np.ndarray:
     if isinstance(starts, list):
         starts = np.array(starts)
@@ -34,7 +32,7 @@ def _slice(
             slices = [slice(s, e) for s, e in zip(starts, ends)]
         else:
             slices = [slice(s, e, d) for s, e, d in zip(starts, ends, steps)]
-    else:
+    else:  # noqa: PLR5501
         if steps is None:
             slices = [slice(0, a) for a in data.shape]
             for s, e, a in zip(starts, ends, axes):
