@@ -3,14 +3,21 @@
 // SPDX-License-Identifier: Apache-2.0
 
 #include "onnx/shape_inference/implementation.h"
+
 #include <algorithm>
 #include <fstream>
 #include <list>
+#include <unordered_map>
+#include <unordered_set>
+#include <utility>
+#include <vector>
+
 #include "onnx/checker.h"
 #include "onnx/common/common.h"
 #include "onnx/common/file_utils.h"
 #include "onnx/defs/data_type_utils.h"
 #include "onnx/proto_utils.h"
+#include "onnx/shape_inference/attribute_binder.h"
 #include "onnx/string_utils.h"
 
 namespace ONNX_NAMESPACE {
@@ -35,9 +42,8 @@ std::string GetValueCaseString(const TypeProto& type) {
       return "sparse_tensor_type";
     case TypeProto::ValueCase::VALUE_NOT_SET:
       return "NOT_SET";
-    default:
-      return ONNX_NAMESPACE::to_string(type.value_case());
   }
+  return ONNX_NAMESPACE::to_string(type.value_case());
 }
 
 std::string GetElemTypeString(const TypeProto_Tensor& type) {
@@ -60,6 +66,10 @@ std::string GetElemTypeString(const TypeProto_SparseTensor& type) {
   return ONNX_NAMESPACE::to_string(type.elem_type());
 }
 
+inline bool IsOnnxDomainOp(const NodeProto& node, const std::string& op_type) {
+  return (IsOnnxDomain(node.domain()) && (node.op_type() == op_type));
+}
+
 } // namespace
 
 template <class T>
@@ -197,6 +207,9 @@ void mergeShapesAndTypes(const TypeProto& inferred_type, TypeProto* existing_typ
     mergeShapesAndTypes(
         inferred_type.optional_type().elem_type(), existing_type->mutable_optional_type()->mutable_elem_type());
   } else if (inferred_val_case == TypeProto::kMapType) {
+    if (existing_type->map_type().key_type() == TensorProto::UNDEFINED) {
+      existing_type->mutable_map_type()->set_key_type(inferred_type.map_type().key_type());
+    }
     mergeShapesAndTypes(inferred_type.map_type().value_type(), existing_type->mutable_map_type()->mutable_value_type());
   }
 }
@@ -211,7 +224,7 @@ void GenerateSymbolicShape(TensorTypeProto* inferred_type, SymbolTable& symbol_t
     // set a symbol if it doesn't have dim_value and dim_param
     auto* dim = inferred_type->mutable_shape()->mutable_dim(i);
     if (!dim->has_dim_value() && !dim->has_dim_param()) {
-      dim->set_dim_param(symbol_table.createNew("unk__"));
+      dim->set_dim_param(symbol_table.createNew());
     }
   }
 }
@@ -230,18 +243,107 @@ void MaterializeSymbolicShape(TypeProto* inferred_type, SymbolTable& symbol_tabl
     MaterializeSymbolicShape(inferred_type->mutable_sequence_type()->mutable_elem_type(), symbol_table);
   } else if (inferred_val_case == TypeProto::kOptionalType) {
     MaterializeSymbolicShape(inferred_type->mutable_optional_type()->mutable_elem_type(), symbol_table);
+  } else if (inferred_val_case == TypeProto::kMapType) {
+    MaterializeSymbolicShape(inferred_type->mutable_map_type()->mutable_value_type(), symbol_table);
   } else {
     fail_shape_inference("type case unsupported for symbolic shape inference. inferred=", inferred_val_case);
   }
 }
 
-std::string GetModelLocalFunctionsMapIdentifier(const std::string& domain, const std::string& func_name) {
-  return domain + ":" + func_name;
+std::string GetFunctionIdentifier(const FunctionProto& function) {
+  // Note: Models with IR version < 10 do not have the overload attribute.
+  // However, that will be mapped to an empty identifier.
+  std::string overload = function.overload();
+  if (overload.empty()) {
+    return function.domain() + ":" + function.name();
+  }
+  return function.domain() + ":" + function.name() + ":" + overload;
+}
+
+std::string GetFunctionIdentifier(const NodeProto& node) {
+  // Note: Models with IR version < 10 do not have the overload attribute.
+  // However, that will be mapped to an empty identifier.
+  std::string overload = node.overload();
+  if (overload.empty()) {
+    return node.domain() + ":" + node.op_type();
+  }
+  return node.domain() + ":" + node.op_type() + ":" + overload;
+}
+
+// InferredTypes: abstracts the differences between FunctionProto and GraphProto
+// for inference. For GraphProto, inferred types are stored in the GraphProto
+// but FunctionProto does not have a place to store inferred types. So, we
+// use a temporary vector (for the duration of inference) to store these.
+class InferredTypes {
+ public:
+  explicit InferredTypes(GraphProto* graph = nullptr) : graph_ptr(graph) {}
+
+  TypeProto* Add(const std::string& var_name, const TypeProto& type) {
+    if (graph_ptr != nullptr) {
+      auto* p = graph_ptr->add_value_info();
+      p->set_name(var_name);
+      *p->mutable_type() = type;
+      return p->mutable_type();
+    } else {
+      auto* p = new TypeProto(type);
+      types.push_back(p);
+      return p;
+    }
+  }
+
+  ~InferredTypes() {
+    for (auto* p : types) {
+      delete p;
+    }
+  }
+
+ private:
+  std::vector<TypeProto*> types;
+  GraphProto* graph_ptr;
+  ONNX_DISALLOW_COPY_ASSIGNMENT_AND_MOVE(InferredTypes);
+};
+
+// Initialize a DataValueMap for a called function from the DataValueMap of the caller
+void BindValuesOnCall(
+    const DataValueMap& caller_map,
+    const NodeProto& caller,
+    DataValueMap& callee_map,
+    const FunctionProto& callee) {
+  auto num_inputs = (std::min)(caller.input_size(), callee.input_size());
+  for (int i = 0; i < num_inputs; ++i) {
+    const std::string& actual = caller.input(i);
+    const std::string& formal = callee.input(i);
+    if (!actual.empty()) {
+      auto it = caller_map.find(actual);
+      if (it != caller_map.end()) {
+        callee_map[formal] = it->second;
+      }
+    }
+  }
+}
+
+// Update a DataValueMap for a calling function from the DataValueMap of the callee
+void BindValuesOnReturn(
+    const DataValueMap& callee_map,
+    const FunctionProto& callee,
+    DataValueMap& caller_map,
+    const NodeProto& caller) {
+  auto num_outputs = (std::min)(caller.output_size(), callee.output_size());
+  for (int i = 0; i < num_outputs; ++i) {
+    const std::string& actual = caller.output(i);
+    const std::string& formal = callee.output(i);
+    if (!actual.empty()) {
+      auto it = callee_map.find(formal);
+      if (it != callee_map.end()) {
+        caller_map[actual] = it->second;
+      }
+    }
+  }
 }
 
 class ShapeInferenceImplBase {
  public:
-  void updateType(const std::string& name, TypeProto* inferred_type) {
+  void UpdateType(const std::string& name, TypeProto* inferred_type) {
     if (inferred_type->value_case() == TypeProto::ValueCase::VALUE_NOT_SET) {
       return;
     }
@@ -254,14 +356,10 @@ class ShapeInferenceImplBase {
     // then check for compatibility with the inferred
     // information. Otherwise, initialize it in an empty state.
     auto iter = value_types_by_name.find(name);
-    TypeProto* existing_type = nullptr;
     if (iter != value_types_by_name.end()) {
-      existing_type = iter->second;
+      mergeShapesAndTypes(*inferred_type, iter->second);
     } else {
-      // Create a new value_info if defined type does not exist
-      auto vi = g.add_value_info(); // TODO: clean this up
-      vi->set_name(name);
-      existing_type = vi->mutable_type();
+      value_types_by_name[name] = inferred_types.Add(name, *inferred_type);
       // For undefined output type, update both value_info and output for now
       // Update existing output with undefined type: assign inferred type to it
       iter = undefined_value_types_by_name.find(name);
@@ -269,16 +367,9 @@ class ShapeInferenceImplBase {
         *iter->second = *inferred_type;
       }
     }
-
-    // TODO: cleanup this by merging with previous if-else
-    // Now we can merge pre-existing and inferred info
-    mergeShapesAndTypes(*inferred_type, existing_type);
-
-    // Make merged info available to further inference.
-    value_types_by_name[name] = existing_type;
   }
 
-  void updateType(ValueInfoProto& valueInfo) {
+  void UpdateType(ValueInfoProto& valueInfo) {
     if (valueInfo.has_type()) {
       value_types_by_name[valueInfo.name()] = valueInfo.mutable_type();
     } else {
@@ -287,15 +378,13 @@ class ShapeInferenceImplBase {
   }
 
   template <typename T>
-  void addTemporaryConstant(const std::string& name, const T& vector) {
+  void AddTemporaryConstant(const std::string& name, const T& vector) {
     input_data_by_name_holder[name] = ToTensor(vector);
     input_data_by_name[name] = &input_data_by_name_holder[name];
   }
 
-  void preprocess(const NodeProto& n) {
-    if (checker::check_is_experimental_op(n)) {
-      has_experimental_op = true;
-    } else if (n.op_type() == "Constant" && n.output().size() == 1) {
+  void ProcessConstant(const NodeProto& n) {
+    if (IsOnnxDomainOp(n, "Constant") && n.output().size() == 1) {
       const std::string& output_name = n.output(0);
       for (const auto& attr : n.attribute()) {
         if (attr.name() == "value") {
@@ -315,22 +404,22 @@ class ShapeInferenceImplBase {
           switch (attr.type()) {
             case AttributeProto::INTS: {
               std::vector<int64_t> ints{attr.ints().begin(), attr.ints().end()};
-              addTemporaryConstant(output_name, ints);
+              AddTemporaryConstant(output_name, ints);
               break;
             }
             case AttributeProto::INT: {
               std::vector<int64_t> ints({attr.i()});
-              addTemporaryConstant(output_name, ints);
+              AddTemporaryConstant(output_name, ints);
               break;
             }
             case AttributeProto::FLOATS: {
               std::vector<float> floats{attr.floats().begin(), attr.floats().end()};
-              addTemporaryConstant(output_name, floats);
+              AddTemporaryConstant(output_name, floats);
               break;
             }
             case AttributeProto::FLOAT: {
               std::vector<float> floats({attr.f()});
-              addTemporaryConstant(output_name, floats);
+              AddTemporaryConstant(output_name, floats);
               break;
             }
             default:
@@ -341,69 +430,31 @@ class ShapeInferenceImplBase {
     }
   }
 
-  // Initialize a DataValueMap for a called function from the DataValueMap of the caller
-  void bindValuesOnCall(
-      const DataValueMap& caller_map,
-      const NodeProto& caller,
-      DataValueMap& callee_map,
-      const FunctionProto& callee) {
-    auto num_inputs = (std::min)(caller.input_size(), callee.input_size());
-    for (int i = 0; i < num_inputs; ++i) {
-      const std::string& actual = caller.input(i);
-      const std::string& formal = callee.input(i);
-      if (!actual.empty()) {
-        auto it = caller_map.find(actual);
-        if (it != caller_map.end()) {
-          callee_map[formal] = it->second;
-        }
-      }
-    }
-  }
-
-  // Update a DataValueMap for a calling function from the DataValueMap of the callee
-  void bindValuesOnReturn(
-      const DataValueMap& callee_map,
-      const FunctionProto& callee,
-      DataValueMap& caller_map,
-      const NodeProto& caller) {
-    auto num_outputs = (std::min)(caller.output_size(), callee.output_size());
-    for (int i = 0; i < num_outputs; ++i) {
-      const std::string& actual = caller.output(i);
-      const std::string& formal = callee.output(i);
-      if (!actual.empty()) {
-        auto it = callee_map.find(formal);
-        if (it != callee_map.end()) {
-          caller_map[actual] = it->second;
-        }
-      }
-    }
-  }
-
-  void processCall(const NodeProto& caller, const FunctionProto& callee, InferenceContext& ctx) {
+  void ProcessCall(const NodeProto& caller, const FunctionProto& callee, InferenceContext& ctx) {
     DataValueMap callee_value_map;
     if (generated_shape_data_by_name != nullptr) {
-      bindValuesOnCall(*generated_shape_data_by_name, caller, callee_value_map, callee);
+      BindValuesOnCall(*generated_shape_data_by_name, caller, callee_value_map, callee);
     }
     InferShapeForFunctionNode(
         callee, schema_registry, ctx, options, model_local_functions_map, symbol_table, &callee_value_map);
     if (generated_shape_data_by_name != nullptr) {
-      bindValuesOnReturn(callee_value_map, callee, *generated_shape_data_by_name, caller);
+      BindValuesOnReturn(callee_value_map, callee, *generated_shape_data_by_name, caller);
     }
   }
 
-  void process(NodeProto& n) {
+  void Process(NodeProto& n) {
     // Resolve domain for node
     auto dit = opset_imports.find(n.domain());
     if (dit == opset_imports.end()) {
-      // Both "" and "ai.onnx" refer to the default ONNX domain
-      if (n.domain() == "") {
-        dit = opset_imports.find("ai.onnx");
+      // Both "" (ONNX_DOMAIN) and "ai.onnx" (AI_ONNX_DOMAIN) refer to the default ONNX domain
+      if (n.domain() == ONNX_DOMAIN) {
+        dit = opset_imports.find(AI_ONNX_DOMAIN);
       }
       if (dit == opset_imports.end()) {
         fail_type_inference(
             "Cannot infer type and shape for node name ",
             n.name(),
-            ". No opset import for domain",
+            ". No opset import for domain ",
             n.domain(),
             " optype ",
             n.op_type());
@@ -416,6 +467,7 @@ class ShapeInferenceImplBase {
         value_types_by_name,
         input_data_by_name,
         input_sparse_data_by_name,
+        options,
         generated_shape_data_by_name,
         &graph_inference_context);
 
@@ -424,15 +476,16 @@ class ShapeInferenceImplBase {
         if (schema->has_type_and_shape_inference_function()) {
           schema->GetTypeAndShapeInferenceFunction()(ctx);
         } else if (schema->HasFunction()) {
-          processCall(n, *(schema->GetFunction()), ctx);
-        } else {
-          // Continue with inference for remaining nodes
-          return;
+          ProcessCall(n, *(schema->GetFunction()), ctx);
+        } // else: rely on schema->CheckInputOutputType() down below.
+        // check type-constraints specified via type variables
+        if (options.check_type) {
+          schema->CheckInputOutputType(ctx);
         }
       } else if (model_local_functions_map.size() > 0) {
-        auto iter = model_local_functions_map.find(GetModelLocalFunctionsMapIdentifier(n.domain(), n.op_type()));
+        auto iter = model_local_functions_map.find(GetFunctionIdentifier(n));
         if (iter != model_local_functions_map.end()) {
-          processCall(n, *(iter->second), ctx);
+          ProcessCall(n, *(iter->second), ctx);
         } else {
           has_unsupported_op = true;
           return;
@@ -441,34 +494,15 @@ class ShapeInferenceImplBase {
         has_unsupported_op = true;
         return;
       }
-    }
-    ONNX_CATCH(const ONNX_NAMESPACE::InferenceError& ex) {
-      ONNX_HANDLE_EXCEPTION([&]() {
-        // onnx does not support unsupported/experimental operators
-        // so it won't consider it as an error
-        if (!has_unsupported_op && !has_experimental_op) {
-          inference_errors.push_back(GetErrorWithNodeInfo(n, ex));
-        }
-      });
-      // Continue with inference for remaining nodes
-      return;
-    }
-
-    ONNX_TRY {
-      // check the type-equality for input and output
-      if (options.check_type && schema) {
-        schema->CheckInputOutputType(ctx);
-      }
-
       for (int i = 0; i < n.output_size(); ++i) {
         // skip type and shape propagation for missing optional outputs.
         if (!n.output(i).empty())
-          updateType(n.output(i), ctx.getOutputType(i));
+          UpdateType(n.output(i), ctx.getOutputType(i));
       }
-
-      preprocess(n);
-
-      // If data propagation is enabled, propagate shape data if it exists.
+      // Constant values are tracked to improve inference/checking for subsequent nodes.
+      ProcessConstant(n);
+      // If data-propagation is enabled, partial-evaluation (aka data-propagation) is performed
+      // to improve inference/checking for subsequent nodes.
       if (options.enable_data_propagation && schema && schema->has_data_propagation_function()) {
         if (generated_shape_data_by_name == nullptr) {
           fail_shape_inference(
@@ -479,14 +513,32 @@ class ShapeInferenceImplBase {
         schema->GetDataPropagationFunction()(data_propagation_ctx);
       }
     }
+    ONNX_CATCH(const ONNX_NAMESPACE::InferenceError& ex) {
+      ONNX_HANDLE_EXCEPTION([&]() {
+        // Note: The following special handling is to accommodate custom-ops. Ideally, custom-ops
+        // should be registered with a schema in the schema registry, allowing inference to handle
+        // them. As things stand, this special handling is somewhat fragile and is not fully
+        // general either. Eg., a custom-op suppresses error-messages for subsequent nodes, but
+        // this does not work across graphs. If special handling is required, a user-option may
+        // be a better way to do it. The fragility comes from the fact that the types of the
+        // returned-values of the custom-op are unknown, and subsequent node-level inference
+        // may fail because of this.
+        if (!has_unsupported_op) {
+          inference_errors.push_back(GetErrorWithNodeInfo(n, ex));
+        }
+      });
+    }
     ONNX_CATCH(const std::runtime_error& err) {
+      // TODO: Fix this. Unclear if this should be remapped to a shape inference error.
+      // Need to rationalize the different types of exceptions that can be thrown.
+      // See: https://github.com/onnx/onnx/pull/5519
       ONNX_HANDLE_EXCEPTION([&]() { fail_shape_inference(GetErrorWithNodeInfo(n, err)); });
     }
   }
 
   // TypeProto_Tensor or TypeProto_SparseTensor
   template <typename T>
-  void processInitializer(
+  void ProcessInitializer(
       const std::string& name,
       const T& tensorValue,
       TypeProto& initializer_type,
@@ -508,18 +560,18 @@ class ShapeInferenceImplBase {
     }
   }
 
-  void process(GraphProto& graph) {
+  void Process(GraphProto& graph) {
     if (symbol_table) {
       TraverseGraphsToAddExistingSymbols(graph, *symbol_table);
     }
     for (auto& vi : *graph.mutable_value_info()) {
-      updateType(vi);
+      UpdateType(vi);
     }
     for (auto& vi : *graph.mutable_input()) {
-      updateType(vi);
+      UpdateType(vi);
     }
     for (auto& vi : *graph.mutable_output()) {
-      updateType(vi);
+      UpdateType(vi);
     }
     for (const auto& tp : graph.initializer()) {
       TypeProto initializer_type;
@@ -530,7 +582,7 @@ class ShapeInferenceImplBase {
       for (int i = 0; i < tp.dims_size(); ++i) {
         shape->add_dim()->set_dim_value(tp.dims(i));
       }
-      processInitializer(tp.name(), tp, initializer_type, input_data_by_name);
+      ProcessInitializer(tp.name(), tp, initializer_type, input_data_by_name);
     }
     for (const auto& tp : graph.sparse_initializer()) {
       TypeProto initializer_type;
@@ -541,66 +593,20 @@ class ShapeInferenceImplBase {
       for (int i = 0; i < tp.dims_size(); ++i) {
         shape->add_dim()->set_dim_value(tp.dims(i));
       }
-      processInitializer(tp.values().name(), tp, initializer_type, input_sparse_data_by_name);
+      ProcessInitializer(tp.values().name(), tp, initializer_type, input_sparse_data_by_name);
     }
     for (auto& n : *graph.mutable_node()) {
-      process(n);
-    }
-    // Throw shape inference error if any. Error mode right now only supports 0 and 1.
-    // When set to 0, any node level shape inference errors are not thrown. This is to support backward compatiblity
-    // with 1.7 and earlier releases. When set to 1 it will throw all exceptions.
-    // TODO: Add a more granular way for exception handling.
-    if (options.error_mode > 0 && !inference_errors.empty()) {
-      std::string full_errors = "Shape inference error(s): ";
-      for (const std::string& error : inference_errors) {
-        full_errors += error + "\n";
-      }
-      fail_shape_inference(full_errors);
+      Process(n);
     }
   }
 
-  void replaceAttrRefs(NodeProto& n, const std::unordered_map<std::string, const AttributeProto*>& attr_map) {
-    auto& attributes = *n.mutable_attribute();
-    for (auto attr_iter = attributes.begin(); attr_iter != attributes.end();) {
-      auto& attr = *attr_iter;
-      if (!attr.ref_attr_name().empty()) {
-        // Attribute-references must be replaced by the corresponding attribute-value in the call-node
-        // if the call-node contains the attribute. Otherwise, this attribute must be removed.
-        auto entry = attr_map.find(attr.ref_attr_name());
-        if (entry != attr_map.cend()) {
-          // Copy value of attribute, but retain original name:
-          std::string name = attr.name();
-          attr = *(entry->second);
-          attr.set_name(name);
-        } else {
-          attr_iter = attributes.erase(attr_iter);
-          continue;
-        }
-      }
-      // Subgraphs must be recursively processed.
-      if (attr.has_g()) {
-        replaceAttrRefs(*attr.mutable_g(), attr_map);
-      }
-      for (auto& graph : *attr.mutable_graphs()) {
-        replaceAttrRefs(graph, attr_map);
-      }
-      ++attr_iter;
-    }
-  }
-
-  void replaceAttrRefs(GraphProto& graph, const std::unordered_map<std::string, const AttributeProto*>& attr_map) {
-    for (auto& n : *graph.mutable_node()) {
-      replaceAttrRefs(n, attr_map);
-    }
-  }
-
-  void process(const NodeProto& n, const std::unordered_map<std::string, const AttributeProto*>& attr_map) {
+  void Process(const NodeProto& n, internal::AttributeBinder& attribute_binder) {
     NodeProto copy_n(n);
-    replaceAttrRefs(copy_n, attr_map);
-    process(copy_n);
+    attribute_binder.VisitNode(&copy_n);
+    Process(copy_n);
   }
 
-  void process(const FunctionProto& func_proto, InferenceContext& ctx) {
+  void Process(const FunctionProto& func_proto, InferenceContext& ctx) {
     // Ensure Constant node tensor-attributes are copied
     bool old_reuse_constant_tensors = reuse_constant_tensors;
     reuse_constant_tensors = false;
@@ -647,8 +653,9 @@ class ShapeInferenceImplBase {
       attr_map[name] = (value != nullptr) ? value : &default_value;
     }
 
+    internal::AttributeBinder attribute_binder(attr_map);
     for (auto& n : func_proto.node()) {
-      process(n, attr_map);
+      Process(n, attribute_binder);
     }
 
     for (int i = 0; i < func_proto.output_size(); ++i) {
@@ -668,7 +675,7 @@ class ShapeInferenceImplBase {
 
  public:
   ShapeInferenceImplBase(
-      GraphProto* g_in,
+      GraphProto* graph, // nullptr for FunctionProto inference
       const std::unordered_map<std::string, TypeProto*>& outer_scope_value_types_by_name_in,
       const std::unordered_map<std::string, int>& opset_imports_in,
       const ShapeInferenceOptions& options_in,
@@ -678,7 +685,7 @@ class ShapeInferenceImplBase {
       DataValueMap* generated_shape_data_by_name_in = nullptr,
       const int ir_version_in = IR_VERSION // default the latest one
       )
-      : g(*g_in),
+      : inferred_types(graph),
         value_types_by_name(outer_scope_value_types_by_name_in),
         opset_imports(opset_imports_in),
         options(options_in),
@@ -701,12 +708,27 @@ class ShapeInferenceImplBase {
     }
   }
 
+  void FinalizeShapeInference() {
+    auto& errors = getErrors();
+    // Throw shape inference error if any. Error mode right now only supports 0 and 1.
+    // When set to 0, any node level shape inference errors are not thrown. This is to support backward compatiblity
+    // with 1.7 and earlier releases. When set to 1 it will throw all exceptions.
+    // TODO: Add a more granular way for exception handling.
+    if (!errors.empty() && options.error_mode > 0) {
+      std::string full_errors = "Inference error(s): ";
+      for (const std::string& error : inference_errors) {
+        full_errors += error + "\n";
+      }
+      fail_shape_inference(full_errors);
+    }
+  }
+
   const std::vector<std::string>& getErrors() const {
     return inference_errors;
   }
 
  private:
-  GraphProto& g;
+  InferredTypes inferred_types;
   std::unordered_map<std::string, TypeProto*> value_types_by_name;
   const std::unordered_map<std::string, int>& opset_imports;
 
@@ -723,7 +745,6 @@ class ShapeInferenceImplBase {
   std::unordered_map<std::string, TensorProto> input_data_by_name_holder;
   std::unordered_map<std::string, const SparseTensorProto*> input_sparse_data_by_name;
 
-  bool has_experimental_op = false;
   bool has_unsupported_op = false;
 
   std::vector<std::string> inference_errors;
@@ -760,7 +781,8 @@ static void InferShapesImpl(
       schema_registry,
       generated_shape_data_by_name,
       ir_version);
-  base.process(*g);
+  base.Process(*g);
+  base.FinalizeShapeInference();
 }
 
 // Either ModelProto or FunctionProto
@@ -799,8 +821,7 @@ void InferShapes(
   SymbolTableImpl symbol_table;
   ModelLocalFunctionsMap model_local_functions_by_id;
   for (const auto& function_proto : m.functions()) {
-    model_local_functions_by_id.insert(
-        {GetModelLocalFunctionsMapIdentifier(function_proto.domain(), function_proto.name()), &function_proto});
+    model_local_functions_by_id.insert({GetFunctionIdentifier(function_proto), &function_proto});
   }
   InferShapesImpl(
       m.mutable_graph(),
@@ -846,9 +867,8 @@ void InferShapeForFunctionNode(
     const std::unordered_map<std::string, const FunctionProto*>& model_local_functions_map,
     SymbolTable* symbol_table,
     DataValueMap* generated_shape_data_by_name) {
-  GraphProto g;
   ShapeInferenceImplBase base(
-      &g,
+      nullptr, // no graph
       {}, // outer_scope_value_types_by_name
       func_opset_imports,
       options,
@@ -856,7 +876,8 @@ void InferShapeForFunctionNode(
       model_local_functions_map,
       schema_registry,
       generated_shape_data_by_name);
-  base.process(func_proto, ctx);
+  base.Process(func_proto, ctx);
+  base.FinalizeShapeInference();
 }
 
 void InferShapeForFunctionNode(
@@ -883,8 +904,9 @@ struct FunctionInferenceContext : public InferenceContext {
   FunctionInferenceContext(
       const FunctionProto& func_proto,
       const std::vector<TypeProto>& input_types,
-      const std::vector<AttributeProto>& attributes)
-      : input_types_(input_types) {
+      const std::vector<AttributeProto>& attributes,
+      const ShapeInferenceOptions& options)
+      : input_types_(input_types), options_(options), func_proto_(&func_proto) {
     for (const auto& attr : attributes) {
       attributesByName_[attr.name()] = &attr;
     }
@@ -949,22 +971,38 @@ struct FunctionInferenceContext : public InferenceContext {
     return std::move(output_types_);
   }
 
+  std::string getDisplayName() const override {
+    if (func_proto_ == nullptr)
+      return "";
+    if (func_proto_->domain().empty()) {
+      if (func_proto_->name().empty())
+        return "";
+      return MakeString("function ", func_proto_->name());
+    }
+    if (func_proto_->name().empty())
+      return MakeString("function [", func_proto_->domain(), "]");
+    return MakeString("function ", func_proto_->name(), "[", func_proto_->domain(), "]");
+  }
+
  private:
   const std::vector<TypeProto>& input_types_;
   std::vector<TypeProto> output_types_;
   std::unordered_map<std::string, const AttributeProto*> attributesByName_;
+  ShapeInferenceOptions options_;
+  const FunctionProto* func_proto_;
 };
 
 std::vector<TypeProto> InferFunctionOutputTypes(
     const FunctionProto& function_proto,
     const std::vector<TypeProto>& input_types,
     const std::vector<AttributeProto>& attributes) {
-  FunctionInferenceContext ctx(function_proto, input_types, attributes);
-  auto opset_imports = GetOpsetImportsFromProto(function_proto);
-  GraphProto g;
+  // TODO: if it is desirable for infer_function_output_types to provide check_type, strict_mode, data_prop,
+  // we can add them to the Python API. For now we just assume the default options.
   ShapeInferenceOptions options{true, 1, false};
+  FunctionInferenceContext ctx(function_proto, input_types, attributes, options);
+  auto opset_imports = GetOpsetImportsFromProto(function_proto);
   ShapeInferenceImplBase base(
-      &g,
+      nullptr, // no graph
       {}, // outer_scope_value_types_by_name
       opset_imports,
       options,
@@ -972,15 +1010,8 @@ std::vector<TypeProto> InferFunctionOutputTypes(
       /*model_local_functions_map*/ {},
       /*schema_registry*/ OpSchemaRegistry::Instance(),
       /*generated_shape_data_by_name*/ nullptr);
-  base.process(function_proto, ctx);
-  auto& errors = base.getErrors();
-  if (!errors.empty()) {
-    std::string all_errors = "Inference error(s): ";
-    for (const std::string& error : errors) {
-      all_errors += error + "\n";
-    }
-    fail_shape_inference(all_errors);
-  }
+  base.Process(function_proto, ctx);
+  base.FinalizeShapeInference();
   return ctx.popOutputTypes();
 }
 
@@ -1045,12 +1076,11 @@ std::vector<const TypeProto*> GraphInferencerImpl::doInferencing(
   // future: pass inputData into InferShapes either directly, or indirectly by
   // updating initializers that match subgraph inputs.
   (void)input_data;
-  ShapeInferenceOptions options{};
   InferShapesImpl(
       g_,
       *context_->outer_scope_value_types_by_name, // never null
       context_->opset_imports,
-      options,
+      options_,
       symbol_table,
       context_->model_local_functions,
       context_->schema_registry,
