@@ -1,9 +1,10 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
+from __future__ import annotations
 
 import unittest
-from typing import Callable, List, Optional, Sequence, Tuple
+from typing import Callable, Sequence
 
 import numpy as np
 
@@ -24,25 +25,19 @@ from onnx import (
 
 
 def _load_model(m_def: str) -> ModelProto:
-    """
-    Parses a model from a string representation, including checking the model for correctness
-    """
+    """Parses a model from a string representation, including checking the model for correctness"""
     m = parser.parse_model(m_def)
     checker.check_model(m)
     return m
 
 
 def _prefixed(prefix: str, s: str) -> str:
-    """
-    Prefixes a string (if not empty)
-    """
+    """Prefixes a string (if not empty)"""
     return prefix + s if len(s) > 0 else s
 
 
-def _get_shape(value_info: ValueInfoProto) -> List[int]:
-    """
-    Returns a list of integers representing the shape of the provided ValueInfoProto
-    """
+def _get_shape(value_info: ValueInfoProto) -> list[int]:
+    """Returns a list of integers representing the shape of the provided ValueInfoProto"""
     return [
         value_info.type.tensor_type.shape.dim[d].dim_value
         for d in range(len(value_info.type.tensor_type.shape.dim))
@@ -93,7 +88,7 @@ M2_DEF = """
     {
         C0 = Add(B01, B11)
         C1 = Sub(B11, B21)
-        M1 = Mul(C0, C1)
+        D0 = Mul(C0, C1)
     }
     """
 
@@ -103,12 +98,12 @@ class TestComposeFunctions(unittest.TestCase):
         self,
         m1def: str,
         m2def: str,
-        io_map: List[Tuple[str, str]],
+        io_map: list[tuple[str, str]],
         check_expectations: Callable[[GraphProto, GraphProto, GraphProto], None],
-        inputs: Optional[List[str]] = None,
-        outputs: Optional[List[str]] = None,
-        prefix1: Optional[str] = None,
-        prefix2: Optional[str] = None,
+        inputs: list[str] | None = None,
+        outputs: list[str] | None = None,
+        prefix1: str | None = None,
+        prefix2: str | None = None,
     ) -> None:
         m1, m2 = _load_model(m1def), _load_model(m2def)
         g3 = compose.merge_graphs(
@@ -135,8 +130,7 @@ class TestComposeFunctions(unittest.TestCase):
         check_expectations(m1.graph, m2.graph, m3.graph)
 
     def test_case_connect_all_no_name_collision(self) -> None:
-        """
-        Tests a simple scenario where two models without overlapping names are merged by
+        """Tests a simple scenario where two models without overlapping names are merged by
         connecting all the outputs in the first models to all the inputs in the second model
         """
 
@@ -152,8 +146,7 @@ class TestComposeFunctions(unittest.TestCase):
         self._test_merge_models(M1_DEF, M2_DEF, io_map, check_expectations)
 
     def test_case_connect_same_output_twice(self) -> None:
-        """
-        Tests a scenario where we merge two models by connecting a single output in the first model
+        """Tests a scenario where we merge two models by connecting a single output in the first model
         to all the inputs in the second
         """
 
@@ -170,8 +163,7 @@ class TestComposeFunctions(unittest.TestCase):
         self._test_merge_models(M1_DEF, M2_DEF, io_map, check_expectations)
 
     def test_case_connect_same_output_drop_outputs(self) -> None:
-        """
-        Tests a scenario where we merge two models by connecting a single output in the first model
+        """Tests a scenario where we merge two models by connecting a single output in the first model
         to all the inputs in the second, while dropping the rest of the outputs in the first model
         """
 
@@ -190,11 +182,9 @@ class TestComposeFunctions(unittest.TestCase):
         )
 
     def test_case_connect_same_input_output_name(self) -> None:
-        """
-        Tests a scenario where we merge two models, where the inputs/outputs connected
+        """Tests a scenario where we merge two models, where the inputs/outputs connected
         are named exactly the same
         """
-
         m1_def = """
             <
                 ir_version: 7,
@@ -226,10 +216,7 @@ class TestComposeFunctions(unittest.TestCase):
         self._test_merge_models(m1_def, m2_def, io_map, check_expectations)
 
     def test_case_drop_inputs_outputs(self) -> None:
-        """
-        Tests a scenario where we merge two models, not including some of the inputs/outputs
-        """
-
+        """Tests a scenario where we merge two models, not including some of the inputs/outputs"""
         m1_def = """
             <
                 ir_version: 7,
@@ -268,11 +255,9 @@ class TestComposeFunctions(unittest.TestCase):
         )
 
     def test_case_name_collision_prefix(self) -> None:
-        """
-        Tests a scenario where we merge two models that have name collisions, but they
+        """Tests a scenario where we merge two models that have name collisions, but they
         are avoided by prefixing the models model.
         """
-
         m1_def = """
             <
                 ir_version: 7,
@@ -297,8 +282,7 @@ class TestComposeFunctions(unittest.TestCase):
         )
 
     def test_case_connect_partially_no_name_collision(self) -> None:
-        """
-        Tests a scenario where two models without overlapping names are merged by
+        """Tests a scenario where two models without overlapping names are merged by
         connecting some outputs from the first model to some inputs in the second.
         The remaining inputs/outputs should be present in the combined model
         """
@@ -337,9 +321,7 @@ class TestComposeFunctions(unittest.TestCase):
         self.assertRaises(ValueError, compose.merge_models, m1, m2, io_map=io_map)
 
     def test_error_wrong_input_output_name(self) -> None:
-        """
-        Tests that providing a non existing output/input name in the io_map argument produces an error.
-        """
+        """Tests that providing a non existing output/input name in the io_map argument produces an error."""
         m1, m2 = _load_model(M1_DEF), _load_model(M2_DEF)
 
         self.assertRaises(
@@ -409,7 +391,7 @@ class TestComposeFunctions(unittest.TestCase):
         checker.check_model(m3)
 
     # FIXME: This function should be removed, as tests should not contain a copy of the tested logic.
-    def _test_add_prefix(  # pylint: disable=too-many-branches
+    def _test_add_prefix(
         self,
         rename_nodes: bool = False,
         rename_edges: bool = False,
@@ -526,33 +508,23 @@ class TestComposeFunctions(unittest.TestCase):
                     self.assertEqual(_prefixed(prefix, n0.name), n1.name)
 
     def test_add_prefix_nodes(self) -> None:
-        """
-        Tests renaming nodes only
-        """
+        """Tests renaming nodes only"""
         self._test_add_prefix(rename_nodes=True)
 
     def test_add_prefix_edges(self) -> None:
-        """
-        Tests prefixing nodes edges. This will also rename inputs/outputs, since the names are shared
-        """
+        """Tests prefixing nodes edges. This will also rename inputs/outputs, since the names are shared"""
         self._test_add_prefix(rename_edges=True)
 
     def test_add_prefix_inputs(self) -> None:
-        """
-        Tests prefixing graph inputs only. Relevant node edges should be renamed as well
-        """
+        """Tests prefixing graph inputs only. Relevant node edges should be renamed as well"""
         self._test_add_prefix(rename_inputs=True)
 
     def test_add_prefix_outputs(self) -> None:
-        """
-        Tests prefixing graph outputs only. Relevant node edges should be renamed as well
-        """
+        """Tests prefixing graph outputs only. Relevant node edges should be renamed as well"""
         self._test_add_prefix(rename_outputs=True)
 
     def test_add_prefix_attribute_subgraph(self) -> None:
-        """
-        Tests prefixing attribute's subgraph. Relevant subgraph should be renamed as well
-        """
+        """Tests prefixing attribute's subgraph. Relevant subgraph should be renamed as well"""
         C = helper.make_tensor_value_info("C", TensorProto.BOOL, [1])
         X = helper.make_tensor_value_info("X", TensorProto.FLOAT, [None, 1])
         Y = helper.make_tensor_value_info("Y", TensorProto.FLOAT, [None, 1])
@@ -597,20 +569,15 @@ class TestComposeFunctions(unittest.TestCase):
                             self.assertEqual(_prefixed(prefix, output_n0), output_n1)
 
     def test_add_prefix_all(self) -> None:
-        """
-        Tests prefixing all names in the graph
-        """
+        """Tests prefixing all names in the graph"""
         self._test_add_prefix(True, True, True, True, True, True)
 
     def test_add_prefix_inplace(self) -> None:
-        """
-        Tests prefixing inplace
-        """
+        """Tests prefixing inplace"""
         self._test_add_prefix(inplace=True)
 
     def test_expand_out_dim(self) -> None:
-        """
-        Tests expanding output dimensions. The resulting graph should have the same output names,
+        """Tests expanding output dimensions. The resulting graph should have the same output names,
         but with one more dimension at the specified index.
         """
         m1 = _load_model(M1_DEF)
@@ -738,24 +705,24 @@ class TestComposeFunctions(unittest.TestCase):
         overlapping_inputs = list(set(inputs0) & set(inputs1))
         overlapping_outputs = list(set(outputs0) & set(outputs1))
         overlapping_edges = list(set(overlapping_inputs + overlapping_outputs))
-        if len(overlapping_edges) > 0:
+        if overlapping_edges:
             self.assertEqual(overlap[i], ("edge", overlapping_edges))
             i += 1
 
         overlapping_vis = list(set(value_info0) & set(value_info1))
-        if len(overlapping_vis) > 0:
+        if overlapping_vis:
             self.assertEqual(overlap[i], ("value_info", overlapping_vis))
             i += 1
 
         overlapping_init = list(set(initializer0) & set(initializer1))
-        if len(overlapping_init) > 0:
+        if overlapping_init:
             self.assertEqual(overlap[i], ("initializer", overlapping_init))
             i += 1
 
         overlapping_sparse_init = list(
             set(sparse_initializer0) & set(sparse_initializer1)
         )
-        if len(overlapping_sparse_init) > 0:
+        if overlapping_sparse_init:
             expected_overlap = []
             for overlapping_name in overlapping_sparse_init:
                 expected_overlap.append(overlapping_name + "_values")
@@ -768,54 +735,42 @@ class TestComposeFunctions(unittest.TestCase):
         self.assertEqual(0, len(overlap))
 
     def test_overlapping_input_names(self) -> None:
-        """
-        Tests error checking when the name of the inputs overlaps
-        """
+        """Tests error checking when the name of the inputs overlaps"""
         self._test_overlapping_names(inputs0=["i0", "i1"], inputs1=["i1", "i2"])
 
     def test_overlapping_output_names(self) -> None:
-        """
-        Tests error checking when the name of the output overlaps
-        """
+        """Tests error checking when the name of the output overlaps"""
         self._test_overlapping_names(outputs0=["o0", "o1"], outputs1=["o1", "o2"])
 
     def test_overlapping_value_info_names(self) -> None:
-        """
-        Tests error checking when the name of value_info entries overlaps
-        """
+        """Tests error checking when the name of value_info entries overlaps"""
         self._test_overlapping_names(
             value_info0=["vi0", "vi1"], value_info1=["vi1", "vi2"]
         )
 
     def test_overlapping_initializer_names(self) -> None:
-        """
-        Tests error checking when the name of initializer entries overlaps
-        """
+        """Tests error checking when the name of initializer entries overlaps"""
         self._test_overlapping_names(
             initializer0=["init0", "init1"], initializer1=["init1", "init2"]
         )
 
     def test_overlapping_sparse_initializer_names(self) -> None:
-        """
-        Tests error checking when the name of sparse_initializer entries overlaps
-        """
+        """Tests error checking when the name of sparse_initializer entries overlaps"""
         self._test_overlapping_names(
             sparse_initializer0=["sparse_init0", "sparse_init1"],
             sparse_initializer1=["sparse_init1", "sparse_init2"],
         )
 
     def test_overlapping_function_names(self) -> None:
-        """
-        Tests error checking when the name of local function entries overlaps
-        """
+        """Tests error checking when the name of local function entries overlaps"""
         ops = [helper.make_opsetid("", 10), helper.make_opsetid("local", 10)]
 
         def _make_function(
             domain: str,
             fname: str,
-            inputs: List[str],
-            outputs: List[str],
-            nodes: List[NodeProto],
+            inputs: list[str],
+            outputs: list[str],
+            nodes: list[NodeProto],
         ) -> FunctionProto:
             f = FunctionProto()
             f.domain = domain
@@ -944,9 +899,7 @@ class TestComposeFunctions(unittest.TestCase):
         )
 
     def test_merge_drop_unnecessary_initializers_and_value_info(self) -> None:
-        """
-        Tests automatic removal of initializers when merging graphs
-        """
+        """Tests automatic removal of initializers when merging graphs"""
         ops = [helper.make_opsetid("", 10)]
 
         g = GraphProto()
