@@ -1,6 +1,7 @@
 # SPDX-License-Identifier: Apache-2.0
 
 # Copyright (c) ONNX Project Contributors
+from __future__ import annotations
 
 import unittest
 
@@ -25,8 +26,7 @@ class TestFunction(unittest.TestCase):
             )
 
     def test_extract_model_with_local_function(self) -> None:
-        r"""
-        #   1. build a model with graph below. extract models with output combinations
+        r"""#   1. build a model with graph below. extract models with output combinations
         #   2. validate extracted models' local functions
         #
         # model graph:
@@ -53,7 +53,6 @@ class TestFunction(unittest.TestCase):
         #                c
         #
         """
-
         # function common
         func_domain = "local"
         func_opset_imports = [onnx.helper.make_opsetid("", 14)]
